@@ -20,14 +20,16 @@
 */
 
 /*! \file cdbase.c
-    \brief Dummy and ISO, BIN/CUE, MDS CD Interfaces
+    \brief Dummy and ISO, BIN/CUE, MDS, CCD CD Interfaces
 */
 
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <ctype.h>
 #include <wchar.h>
 #include "cdbase.h"
+#include "cs2.h"
 #include "error.h"
 #include "debug.h"
 
@@ -58,10 +60,18 @@ static char * wcsdupstr(const wchar_t * path)
 static FILE * _wfopen(const wchar_t *wpath, const wchar_t *wmode)
 {
    FILE * fd;
-   char * path = wcsdupstr(wpath);
-   char * mode = wcsdupstr(wmode);
+   char * path;
+   char * mode;
 
-   if ((path == NULL) || (mode == NULL)) return NULL;
+   path = wcsdupstr(wpath);
+   if (path == NULL) return NULL;
+
+   mode = wcsdupstr(wmode);
+   if (mode == NULL)
+   {
+      free(path);
+      return NULL;
+   }
 
    fd = fopen(path, mode);
 
@@ -80,6 +90,7 @@ static int DummyCDInit(const char *);
 static void DummyCDDeInit(void);
 static int DummyCDGetStatus(void);
 static s32 DummyCDReadTOC(u32 *);
+static s32 DummyCDReadTOC10(CDInterfaceToc10 *);
 static int DummyCDReadSectorFAD(u32, void *);
 static void DummyCDReadAheadFAD(u32);
 
@@ -90,6 +101,7 @@ DummyCDInit,
 DummyCDDeInit,
 DummyCDGetStatus,
 DummyCDReadTOC,
+DummyCDReadTOC10,
 DummyCDReadSectorFAD,
 DummyCDReadAheadFAD,
 };
@@ -98,6 +110,7 @@ static int ISOCDInit(const char *);
 static void ISOCDDeInit(void);
 static int ISOCDGetStatus(void);
 static s32 ISOCDReadTOC(u32 *);
+static s32 ISOCDReadTOC10(CDInterfaceToc10 *);
 static int ISOCDReadSectorFAD(u32, void *);
 static void ISOCDReadAheadFAD(u32);
 
@@ -108,6 +121,7 @@ ISOCDInit,
 ISOCDDeInit,
 ISOCDGetStatus,
 ISOCDReadTOC,
+ISOCDReadTOC10,
 ISOCDReadSectorFAD,
 ISOCDReadAheadFAD,
 };
@@ -195,6 +209,13 @@ static s32 DummyCDReadTOC(UNUSED u32 *TOC)
 
 //////////////////////////////////////////////////////////////////////////////
 
+static s32 DummyCDReadTOC10(UNUSED CDInterfaceToc10 *TOC)
+{
+	return 0;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 static int DummyCDReadSectorFAD(UNUSED u32 FAD, void * buffer)
 {
 	// This function is supposed to read exactly 1 -RAW- 2352-byte sector
@@ -245,6 +266,7 @@ typedef struct
    u32 file_offset;
    u32 sector_size;
    FILE *fp;
+	FILE *sub_fp;
    int file_size;
    int file_id;
    int interleaved_sub;
@@ -328,10 +350,29 @@ typedef struct
 
 #pragma pack(pop)
 
+#define CCD_MAX_SECTION 20
+#define CCD_MAX_NAME 30
+#define CCD_MAX_VALUE 20
+
+typedef struct
+{
+	char section[CCD_MAX_SECTION];
+	char name[CCD_MAX_NAME];
+	char value[CCD_MAX_VALUE];
+} ccd_dict_struct;
+
+typedef struct
+{
+	ccd_dict_struct *dict;
+	int num_dict;
+} ccd_struct;
+
 static const s8 syncHdr[12] = { 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00 };
 enum IMG_TYPE { IMG_NONE, IMG_ISO, IMG_BINCUE, IMG_MDS, IMG_CCD, IMG_NRG };
 enum IMG_TYPE imgtype = IMG_ISO;
 static u32 isoTOC[102];
+static CDInterfaceToc10 isoTOC10[102];
+int isoTOCnum=0;
 static disc_info_struct disc;
 
 #define MSF_TO_FAD(m,s,f) ((m * 4500) + (s * 75) + f)
@@ -340,7 +381,7 @@ static disc_info_struct disc;
 
 static int LoadBinCue(const char *cuefilename, FILE *iso_file)
 {
-   u32 size;
+   long size;
    char *temp_buffer, *temp_buffer2;
    unsigned int track_num;
    unsigned int indexnum, min, sec, frame;
@@ -350,7 +391,9 @@ static int LoadBinCue(const char *cuefilename, FILE *iso_file)
    int file_size;
    int i;
    FILE * bin_file;
+   int matched = 0;
 
+	memset(trk, 0, sizeof(trk));
    disc.session_num = 1;
    disc.session = malloc(sizeof(session_info_struct) * disc.session_num);
    if (disc.session == NULL)
@@ -361,6 +404,13 @@ static int LoadBinCue(const char *cuefilename, FILE *iso_file)
 
    fseek(iso_file, 0, SEEK_END);
    size = ftell(iso_file);
+
+   if(size <= 0)
+   {
+      YabSetError(YAB_ERR_FILEREAD, cuefilename);
+      return -1;
+   }
+
    fseek(iso_file, 0, SEEK_SET);
 
    // Allocate buffer with enough space for reading cue
@@ -441,7 +491,7 @@ static int LoadBinCue(const char *cuefilename, FILE *iso_file)
 
    // Go back, retrieve image filename
    fseek(iso_file, 0, SEEK_SET);
-   fscanf(iso_file, "FILE \"%[^\"]\" %*s\r\n", temp_buffer);
+   matched = fscanf(iso_file, "FILE \"%[^\"]\" %*s\r\n", temp_buffer);
 
    // Now go and open up the image file, figure out its size, etc.
    if ((bin_file = fopen(temp_buffer, "rb")) == NULL)
@@ -518,6 +568,7 @@ static int LoadBinCue(const char *cuefilename, FILE *iso_file)
    disc.session[0].fad_start = 150;
    disc.session[0].fad_end = trk[track_num-1].fad_end;
    disc.session[0].track_num = track_num;
+
    disc.session[0].track = malloc(sizeof(track_info_struct) * disc.session[0].track_num);
    if (disc.session[0].track == NULL)
    {
@@ -542,14 +593,15 @@ int LoadMDSTracks(const char *mds_filename, FILE *iso_file, mds_session_struct *
 {
    int i;
    int track_num=0;
-   u32 fad_end;
+   u32 fad_end = 0;
 
-   session->track = malloc(sizeof(mds_track_struct) * mds_session->last_track);
+   session->track = malloc(sizeof(track_info_struct) * mds_session->last_track);
    if (session->track == NULL)
    {
       YabSetError(YAB_ERR_MEMORYALLOC, NULL);
       return -1;
    }
+	memset(session->track, 0, sizeof(track_info_struct) * mds_session->last_track);
 
    for (i = 0; i < mds_session->total_blocks; i++)
    {
@@ -643,6 +695,13 @@ int LoadMDSTracks(const char *mds_filename, FILE *iso_file, mds_session_struct *
                if (strncmp(img_filename, "*.", 2) == 0)
                {
                   char *ext;
+                  size_t mds_filename_len = strlen(mds_filename);
+                  if (mds_filename_len >= 512)
+                  {
+                     YabSetError(YAB_ERR_FILEREAD, mds_filename);
+                     free(session->track);
+                     return -1;
+                  }
                   strcpy(filename, mds_filename);
                   ext = strrchr(filename, '.');
                   strcpy(ext, img_filename+1);
@@ -773,6 +832,8 @@ static int LoadISO(FILE *iso_file)
       return -1;
    }
 
+	memset(disc.session[0].track, 0, sizeof(track_info_struct) * disc.session[0].track_num);
+
    track = disc.session[0].track;
    track->ctl_addr = 0x41;
    track->fad_start = 150;
@@ -799,6 +860,290 @@ static int LoadISO(FILE *iso_file)
 
 //////////////////////////////////////////////////////////////////////////////
 
+char* StripPreSuffixWhitespace(char* string)
+{
+	char* p;
+	for (;;)
+	{
+		if (string[0] == 0 || !isspace(string[0]))
+			break;
+		string++;
+	}
+
+	if (strlen(string) == 0)
+		return string;
+
+	p = string+strlen(string)-1;
+	for (;;)
+	{
+		if (p <= string || !isspace(p[0]))
+		{
+			p[1] = '\0';
+			break;
+		}
+		p--;
+	}
+
+	return string;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
+int LoadParseCCD(FILE *ccd_fp, ccd_struct *ccd)
+{
+	char text[60], section[CCD_MAX_SECTION], old_name[CCD_MAX_NAME] = "";
+	char * start, *end, *name, *value;
+	int lineno = 0, error = 0, max_size = 100;
+
+	ccd->dict = (ccd_dict_struct *)malloc(sizeof(ccd_dict_struct)*max_size);
+	if (ccd->dict == NULL) 
+		return -1;
+
+	ccd->num_dict = 0;
+
+	// Read CCD file
+	while (fgets(text, sizeof(text), ccd_fp) != NULL) 
+	{
+		lineno++;
+
+		start = StripPreSuffixWhitespace(text);
+
+		if (start[0] == '[') 
+		{
+			// Section
+			end = strchr(start+1, ']');
+			if (end == NULL) 
+			{
+				// ] missing from section
+				error = lineno;
+			}
+			else
+			{
+				end[0] = '\0';
+				memset(section, 0, sizeof(section));
+				strncpy(section, start + 1, sizeof(section));
+				old_name[0] = '\0';
+			}
+		}
+		else if (start[0]) 
+		{
+			// Name/Value pair
+			end = strchr(start, '=');
+			if (end) 
+			{
+				end[0] = '\0';
+				name = StripPreSuffixWhitespace(start);
+				value = StripPreSuffixWhitespace(end + 1);
+
+				memset(old_name, 0, sizeof(old_name));
+				strncpy(old_name, name, sizeof(old_name));
+				if (ccd->num_dict+1 > max_size)
+				{
+					max_size *= 2;
+					ccd->dict = realloc(ccd->dict, sizeof(ccd_dict_struct)*max_size);
+					if (ccd->dict == NULL)
+					{
+						free(ccd->dict);
+						return -2;
+					}
+				}
+				strcpy(ccd->dict[ccd->num_dict].section, section);
+				strcpy(ccd->dict[ccd->num_dict].name, name);
+				strcpy(ccd->dict[ccd->num_dict].value, value);
+				ccd->num_dict++;
+			}
+			else
+				error = lineno;
+		}
+
+		if (error)
+			break;
+	}
+
+	if (error)
+	{
+		free(ccd->dict);
+		ccd->num_dict = 0;
+	}
+
+	return error;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
+static int GetIntCCD(ccd_struct *ccd, char *section, char *name)
+{
+	int i;
+	for (i = 0; i < ccd->num_dict; i++)
+	{
+		if (stricmp(ccd->dict[i].section, section) == 0 &&
+			 stricmp(ccd->dict[i].name, name) == 0)
+			return strtol(ccd->dict[i].value, NULL, 0);
+	}
+
+	return -1;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
+static int LoadCCD(const char *ccd_filename, FILE *iso_file)
+{
+	int i;
+	ccd_struct ccd;
+	int num_toc;
+	char img_filename[512];
+	char *ext;
+	FILE *fp;
+   size_t ccd_filename_len = strlen(ccd_filename);
+
+   if (ccd_filename_len >= 512)
+   {
+      YabSetError(YAB_ERR_FILEREAD, ccd_filename);
+      return -1;
+   }
+
+	strcpy(img_filename, ccd_filename);
+	ext = strrchr(img_filename, '.');
+	strcpy(ext, ".img");
+	fp = fopen(img_filename, "rb");
+
+	if (fp == NULL)
+	{
+		YabSetError(YAB_ERR_FILEREAD, img_filename);
+		return -1;
+	}
+
+	fseek(iso_file, 0, SEEK_SET);
+
+	// Load CCD file as dictionary
+	if (LoadParseCCD(iso_file, &ccd))
+	{
+		fclose(fp);
+		YabSetError(YAB_ERR_FILEREAD, ccd_filename);
+		return -1;
+	}
+
+	num_toc = GetIntCCD(&ccd, "DISC", "TocEntries");
+	disc.session_num = GetIntCCD(&ccd, "DISC", "Sessions");
+	if (disc.session_num != 1)
+	{
+		fclose(fp);
+		YabSetError(YAB_ERR_OTHER, "Sessions more than 1 are unsupported");
+		return -1;
+	}
+
+	disc.session = malloc(sizeof(session_info_struct) * disc.session_num);
+	if (disc.session == NULL)
+	{
+		fclose(fp);
+		free(ccd.dict);
+		YabSetError(YAB_ERR_MEMORYALLOC, NULL);
+		return -1;
+	}
+
+	if (GetIntCCD(&ccd, "DISC", "DataTracksScrambled"))
+	{
+		fclose(fp);
+		free(ccd.dict);
+		free(disc.session);
+		YabSetError(YAB_ERR_OTHER, "CCD Scrambled Tracks not supported");
+		return -1;
+	}
+
+	isoTOCnum = num_toc;
+
+	// Find track number and allocate
+	for (i = 0; i < num_toc; i++)
+	{
+		char sect_name[64];
+		int point;
+
+		sprintf(sect_name, "Entry %d", i);
+
+		isoTOC10[i].ctrladr = (GetIntCCD(&ccd, sect_name, "Control") << 4) | GetIntCCD(&ccd, sect_name, "ADR");
+		isoTOC10[i].tno = GetIntCCD(&ccd, sect_name, "TrackNo");
+		isoTOC10[i].point = GetIntCCD(&ccd, sect_name, "Point");
+		isoTOC10[i].min = GetIntCCD(&ccd, sect_name, "AMin");
+		isoTOC10[i].sec = 2;
+		isoTOC10[i].frame = 0;
+		isoTOC10[i].zero = GetIntCCD(&ccd, sect_name, "Zero");
+		isoTOC10[i].pmin = GetIntCCD(&ccd, sect_name, "PMin");
+		isoTOC10[i].psec = GetIntCCD(&ccd, sect_name, "PSec");
+		isoTOC10[i].pframe = GetIntCCD(&ccd, sect_name, "PFrame");
+
+		point = GetIntCCD(&ccd, sect_name, "Point");
+
+		if (point == 0xA1)
+		{
+			int ses = GetIntCCD(&ccd, sect_name, "Session");
+
+			disc.session[ses-1].fad_start = 150;
+			disc.session[ses-1].track_num=GetIntCCD(&ccd, sect_name, "PMin");;
+			disc.session[ses-1].track = (track_info_struct *)malloc(disc.session[ses-1].track_num * sizeof(track_info_struct));
+			if (disc.session[ses-1].track == NULL)
+			{
+				fclose(fp);
+				free(ccd.dict);
+				free(disc.session);
+				YabSetError(YAB_ERR_MEMORYALLOC, NULL);
+				return -1;
+			}
+			memset(disc.session[ses-1].track, 0, disc.session[ses-1].track_num * sizeof(track_info_struct));
+		}
+	}
+
+	// Load TOC
+	for (i = 0; i < num_toc; i++)
+	{
+		char sect_name[64];
+		int ses, point, adr, control, trackno, amin, asec, aframe;
+		int alba, zero, pmin, psec, pframe, plba;
+
+		sprintf(sect_name, "Entry %d", i);
+
+		ses = GetIntCCD(&ccd, sect_name, "Session");
+		point = GetIntCCD(&ccd, sect_name, "Point");
+		adr = GetIntCCD(&ccd, sect_name, "ADR");
+		control = GetIntCCD(&ccd, sect_name, "Control");
+		trackno = GetIntCCD(&ccd, sect_name, "TrackNo");
+		amin = GetIntCCD(&ccd, sect_name, "AMin");
+		asec = GetIntCCD(&ccd, sect_name, "ASec");
+		aframe = GetIntCCD(&ccd, sect_name, "AFrame");
+		alba = GetIntCCD(&ccd, sect_name, "ALBA");
+		zero = GetIntCCD(&ccd, sect_name, "Zero");
+		pmin = GetIntCCD(&ccd, sect_name, "PMin");
+		psec = GetIntCCD(&ccd, sect_name, "PSec");
+		pframe = GetIntCCD(&ccd, sect_name, "PFrame");
+		plba = GetIntCCD(&ccd, sect_name, "PLBA");
+
+		if(point >= 1 && point <= 99)
+		{
+			track_info_struct *track=&disc.session[ses-1].track[point-1];
+			track->ctl_addr = (control << 4) | adr;
+			track->fad_start = MSF_TO_FAD(pmin, psec, pframe);
+			if (point >= 2)
+			   disc.session[ses-1].track[point-2].fad_end = track->fad_start-1;
+			track->file_offset = plba*2352;
+			track->sector_size = 2352;
+			track->fp = fp;
+			track->file_size = (track->fad_end+1-track->fad_start)*2352;
+			track->file_id = 0;
+			track->interleaved_sub = 0;
+		}
+		else if (point == 0xA2)
+		{
+			disc.session[ses-1].fad_end = MSF_TO_FAD(pmin, psec, pframe);
+			disc.session[ses-1].track[disc.session[ses-1].track_num-1].fad_end = disc.session[ses-1].fad_end;
+		}
+	}
+
+	fclose(iso_file);
+
+	return 0;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 void BuildTOC()
 {
    int i;
@@ -817,11 +1162,64 @@ void BuildTOC()
 
 //////////////////////////////////////////////////////////////////////////////
 
+void BuildTOC10()
+{
+   int i;
+   session_info_struct *session=&disc.session[0];
+
+   for (i = 0; i < session->track_num; i++)
+   {
+      isoTOC10[3+i].ctrladr = session->track[i].ctl_addr;
+      isoTOC10[3+i].tno = 0;
+      isoTOC10[3+i].point = i+1;
+      isoTOC10[3+i].min = 0;
+      isoTOC10[3+i].sec = 2;
+      isoTOC10[3+i].frame = 0;
+      isoTOC10[3+i].zero = 0;
+      Cs2FADToMSF(session->track[i].fad_start, &isoTOC10[3+i].pmin, &isoTOC10[3+i].psec, &isoTOC10[3+i].pframe);
+   }
+
+   isoTOC10[0].ctrladr = isoTOC10[3].ctrladr;
+   isoTOC10[0].tno = 0;
+   isoTOC10[0].point = 0xA0;
+   isoTOC10[0].min = 0;
+   isoTOC10[0].sec = 2;
+   isoTOC10[0].frame = 0;
+   isoTOC10[0].zero = 0;
+   isoTOC10[0].pmin = 1;
+   isoTOC10[0].psec = 0;
+   isoTOC10[0].pframe = 0;
+
+   isoTOC10[1].ctrladr = isoTOC10[3+session->track_num-1].ctrladr;
+   isoTOC10[1].tno = 0;
+   isoTOC10[1].point = 0xA1;
+   isoTOC10[1].min = 0;
+   isoTOC10[1].sec = 2;
+   isoTOC10[1].frame = 0;
+   isoTOC10[1].zero = 0;
+   isoTOC10[1].pmin = session->track_num;
+   isoTOC10[1].psec = 0;
+   isoTOC10[1].pframe = 0;
+
+   isoTOC10[2].ctrladr = isoTOC10[1].ctrladr;
+   isoTOC10[2].tno = 0;
+   isoTOC10[2].point = 0xA2;
+   isoTOC10[2].min = 0;
+   isoTOC10[2].sec = 2;
+   isoTOC10[2].frame = 0;
+   isoTOC10[2].zero = 0;
+   Cs2FADToMSF(session->fad_end, &isoTOC10[2].pmin, &isoTOC10[2].psec, &isoTOC10[2].pframe);
+   isoTOCnum = 3+session->track_num;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 static int ISOCDInit(const char * iso) {
    char header[6];
    char *ext;
    int ret;
    FILE *iso_file;
+   size_t num_read = 0;
 
    memset(isoTOC, 0xFF, 0xCC * 2);
    memset(&disc, 0, sizeof(disc));
@@ -835,7 +1233,7 @@ static int ISOCDInit(const char * iso) {
       return -1;
    }
 
-   fread((void *)header, 1, 6, iso_file);
+   num_read = fread((void *)header, 1, 6, iso_file);
    ext = strrchr(iso, '.');
 
    // Figure out what kind of image format we're dealing with
@@ -851,6 +1249,12 @@ static int ISOCDInit(const char * iso) {
       imgtype = IMG_MDS;
       ret = LoadMDS(iso, iso_file);
    }
+	else if (stricmp(ext, ".CCD") == 0)
+	{
+		// It's a CCD
+		imgtype = IMG_CCD;
+		ret = LoadCCD(iso, iso_file);
+	}
    else
    {
       // Assume it's an ISO file
@@ -869,6 +1273,8 @@ static int ISOCDInit(const char * iso) {
    }   
 
    BuildTOC();
+   if (imgtype != IMG_CCD)
+      BuildTOC10();
    return 0;
 }
 
@@ -919,8 +1325,16 @@ static s32 ISOCDReadTOC(u32 * TOC) {
 
 //////////////////////////////////////////////////////////////////////////////
 
+static s32 ISOCDReadTOC10(CDInterfaceToc10 *TOC) {
+   memcpy(TOC, isoTOC10, 102 * sizeof(CDInterfaceToc10));
+   return isoTOCnum;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 static int ISOCDReadSectorFAD(u32 FAD, void *buffer) {
    int i,j;
+   size_t num_read = 0;
    track_info_struct *track=NULL;
 
    assert(disc.session);
@@ -947,10 +1361,20 @@ static int ISOCDReadSectorFAD(u32 FAD, void *buffer) {
    }
 
    fseek(track->fp, track->file_offset + (FAD-track->fad_start) * track->sector_size, SEEK_SET);
+	if (track->sub_fp)
+		fseek(track->sub_fp, track->file_offset + (FAD-track->fad_start) * 96, SEEK_SET);
    if (track->sector_size == 2448)
    {
       if (!track->interleaved_sub)
-         fread(buffer, 2448, 1, track->fp);
+		{
+			if (track->sub_fp)
+			{
+            num_read = fread(buffer, 2352, 1, track->fp);
+            num_read = fread((char *)buffer + 2352, 96, 1, track->sub_fp);
+			}
+			else
+            num_read = fread(buffer, 2448, 1, track->fp);
+		}
       else
       {
          const u16 deint_offsets[] = {
@@ -965,13 +1389,13 @@ static int ISOCDReadSectorFAD(u32 FAD, void *buffer) {
          };
          u8 subcode_buffer[96 * 3];
 
-         fread(buffer, 2352, 1, track->fp);
+         num_read = fread(buffer, 2352, 1, track->fp);
 
-         fread(subcode_buffer, 96, 1, track->fp);
+         num_read = fread(subcode_buffer, 96, 1, track->fp);
          fseek(track->fp, 2352, SEEK_CUR);
-         fread(subcode_buffer+96, 96, 1, track->fp);
+         num_read = fread(subcode_buffer + 96, 96, 1, track->fp);
          fseek(track->fp, 2352, SEEK_CUR);
-         fread(subcode_buffer+192, 96, 1, track->fp);
+         num_read = fread(subcode_buffer + 192, 96, 1, track->fp);
          for (i = 0; i < 96; i++)
             ((u8 *)buffer)[2352+i] = subcode_buffer[deint_offsets[i]];
       }
@@ -979,12 +1403,12 @@ static int ISOCDReadSectorFAD(u32 FAD, void *buffer) {
    else if (track->sector_size == 2352)
    {
       // Generate subcodes here
-      fread(buffer, 2352, 1, track->fp);
+      num_read = fread(buffer, 2352, 1, track->fp);
    }
    else if (track->sector_size == 2048)
    {
       memcpy(buffer, syncHdr, 12);
-      fread((char *)buffer + 0x10, 2048, 1, track->fp);
+      num_read = fread((char *)buffer + 0x10, 2048, 1, track->fp);
    }
 	return 1;
 }
@@ -997,4 +1421,3 @@ static void ISOCDReadAheadFAD(UNUSED u32 FAD)
 }
 
 //////////////////////////////////////////////////////////////////////////////
-
