@@ -241,25 +241,25 @@ void CheatDoPatches(void)
          case CHEATTYPE_ENABLE:
             if (cheatlist[i].enable == 0)
                continue;
-            if (MappedMemoryReadWord(cheatlist[i].addr) != cheatlist[i].val)
+            if (MappedMemoryReadWordNocache(MSH2, cheatlist[i].addr) != cheatlist[i].val)
                return;
             break;
          case CHEATTYPE_BYTEWRITE:
             if (cheatlist[i].enable == 0)
                continue;
-            MappedMemoryWriteByte(cheatlist[i].addr, (u8)cheatlist[i].val);
+            MappedMemoryWriteByteNocache(MSH2, cheatlist[i].addr, (u8)cheatlist[i].val);
             SH2WriteNotify(cheatlist[i].addr, 1);
             break;
          case CHEATTYPE_WORDWRITE:
             if (cheatlist[i].enable == 0)
                continue;
-            MappedMemoryWriteWord(cheatlist[i].addr, (u16)cheatlist[i].val);
+            MappedMemoryWriteWordNocache(MSH2, cheatlist[i].addr, (u16)cheatlist[i].val);
             SH2WriteNotify(cheatlist[i].addr, 2);
             break;
          case CHEATTYPE_LONGWRITE:
             if (cheatlist[i].enable == 0)
                continue;
-            MappedMemoryWriteLong(cheatlist[i].addr, cheatlist[i].val);
+            MappedMemoryWriteLongNocache(MSH2, cheatlist[i].addr, cheatlist[i].val);
             SH2WriteNotify(cheatlist[i].addr, 4);
             break;            
       }
@@ -284,7 +284,7 @@ int CheatSave(const char *filename)
    FILE *fp;
    int i;
    int num;
-   IOCheck_struct check;
+   IOCheck_struct check = { 0, 0 };
 
    if (!filename)
       return -1;
@@ -333,7 +333,7 @@ int CheatLoad(const char *filename)
    int i;
    char id[4];
    char desc[256];
-   IOCheck_struct check;
+   IOCheck_struct check = { 0, 0 };
 
    if (!filename)
       return -1;
