@@ -73,6 +73,12 @@ void LogChangeOutput(DebugOutType t, char * s);
 #define SCSPLOG(...)
 #endif
 
+#ifdef SCSPDSP_DEBUG
+#define SCSPDSPLOG(...) DebugPrintf(MainLog, __FILE__, __LINE__, __VA_ARGS__)
+#else
+#define SCSPDSPLOG(...)
+#endif
+
 #ifdef VDP1_DEBUG
 #define VDP1LOG(...) DebugPrintf(MainLog, __FILE__, __LINE__, __VA_ARGS__)
 #else
@@ -91,4 +97,10 @@ void LogChangeOutput(DebugOutType t, char * s);
 #define SMPCLOG(...)
 #endif
 
+#ifdef YGL_DEBUG
+#define YGLLOG(...) DebugPrintf(MainLog, __FILE__, __LINE__, __VA_ARGS__)
+#else
+#define YGLLOG(...)
+#endif
+
 #endif
