@@ -17,6 +17,10 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+/*! \file error.c
+    \brief Error handling functions.
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
