@@ -91,6 +91,9 @@ static void writeb_trampoline(uint32_t address, uint32_t data);
 static void writew_trampoline(uint32_t address, uint32_t data);
 #endif
 
+static void m68kq68_save_state(FILE * fp);
+static void m68kq68_load_state(FILE * fp);
+
 /*-----------------------------------------------------------------------*/
 
 /* Module interface definition */
@@ -128,6 +131,8 @@ M68K_struct M68KQ68 = {
     .SetReadW    = m68kq68_set_readw,
     .SetWriteB   = m68kq68_set_writeb,
     .SetWriteW   = m68kq68_set_writew,
+    .SaveState   = m68kq68_save_state,
+    .LoadState   = m68kq68_load_state,
 };
 
 /*-----------------------------------------------------------------------*/
@@ -520,6 +525,68 @@ static void writew_trampoline(uint32_t address, uint32_t data) {
 
 #endif  // NEED_TRAMPOLINE
 
+static void m68kq68_save_state(FILE * fp)
+{
+   int i = 0;
+   u32 val = 0;
+   IOCheck_struct check = { 0, 0 };
+
+   for (i = 0; i < 8; i++)
+   {
+      val = q68_get_dreg(state, i);
+      ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+   }
+
+   for (i = 0; i < 8; i++)
+   {
+      val = q68_get_areg(state, i, val);
+      ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+   }
+
+   val = q68_get_pc(state, val);
+   ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+   
+   val = q68_get_sr(state, val);
+   ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+   
+   val = q68_get_usp(state, val);
+   ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+   
+   val = q68_get_ssp(state, val);
+   ywrite(&check, (void *)&val, sizeof(u32), 1, fp);
+}
+
+static void m68kq68_load_state(FILE * fp)
+{
+   int i = 0;
+   u32 val = 0;
+   IOCheck_struct check = { 0, 0 };
+
+   for (i = 0; i < 8; i++)
+   {
+      yread(&check, (void *)&val, sizeof(u32), 1, fp);
+      q68_set_dreg(state, i, val);
+   }
+
+   for (i = 0; i < 8; i++)
+   {
+      yread(&check, (void *)&val, sizeof(u32), 1, fp);
+      q68_set_areg(state, i, val);
+   }
+
+   yread(&check, (void *)&val, sizeof(u32), 1, fp);
+   q68_set_pc(state, val);
+
+   yread(&check, (void *)&val, sizeof(u32), 1, fp);
+   q68_set_sr(state, val);
+
+   yread(&check, (void *)&val, sizeof(u32), 1, fp);
+   q68_set_usp(state, val);
+
+   yread(&check, (void *)&val, sizeof(u32), 1, fp);
+   q68_set_ssp(state, val);
+}
+
 /*************************************************************************/
 /*************************************************************************/
 
