@@ -26,6 +26,7 @@
 #include "cs2.h"
 #include "vdp2.h"  // for DisplayMessage() prototype
 #include "yabause.h"
+#include "error.h"
 
 int RecordingFileOpened;
 int PlaybackFileOpened;
@@ -46,11 +47,12 @@ int headersize=512;
 //////////////////////////////////////////////////////////////////////////////
 
 static void ReadHeader(FILE* fp) {
+   size_t num_read = 0;
 
 	fseek(fp, 0, SEEK_SET);
 
 	fseek(fp, 172, SEEK_SET);
-	fread(&Movie.Rerecords, sizeof(Movie.Rerecords), 1, fp);
+   num_read = fread(&Movie.Rerecords, sizeof(Movie.Rerecords), 1, fp);
 
 	fseek(fp, headersize, SEEK_SET);
 }
@@ -102,20 +104,36 @@ static void SetInputDisplayCharacters(void) {
 	for (x = 0; x < 8; x++) {
 
 		if(PORTDATA1.data[2] & (1 << x)) {
+         size_t spaces_len = strlen(Spaces[x]);
+         if (spaces_len >= 40)
+            return;
 			strcat(str, Spaces[x]);	
 		}
-		else
-			strcat(str, Buttons[x]);
+      else
+      {
+         size_t buttons_len = strlen(Buttons[x]);
+         if (buttons_len >= 40)
+            return;
+         strcat(str, Buttons[x]);
+      }
 
 	}
 
 	for (x = 0; x < 8; x++) {
 
 		if(PORTDATA1.data[3] & (1 << x)) {
+         size_t spaces2_len = strlen(Spaces2[x]);
+         if (spaces2_len >= 40)
+            return;
 			strcat(str, Spaces2[x]);	
 		}
-		else
-			strcat(str, Buttons2[x]);
+      else
+      {
+         size_t buttons2_len = strlen(Buttons2[x]);
+         if (buttons2_len >= 40)
+            return;
+         strcat(str, Buttons2[x]);
+      }
 
 	}
 
@@ -139,6 +157,8 @@ int framelength=16;
 void DoMovie(void) {
 
 	int x;
+   size_t num_read = 0;
+
 	if (Movie.Status == 0)
 		return;
 
@@ -158,10 +178,10 @@ void DoMovie(void) {
 
 	if(Movie.Status == Playback) {
 		for (x = 0; x < 8; x++) {
-			fread(&PORTDATA1.data[x], 1, 1, Movie.fp);
+         num_read = fread(&PORTDATA1.data[x], 1, 1, Movie.fp);
 		}
 		for (x = 0; x < 8; x++) {
-			fread(&PORTDATA2.data[x], 1, 1, Movie.fp);
+         num_read = fread(&PORTDATA2.data[x], 1, 1, Movie.fp);
 		}
 
 		//if we get to the end of the movie
@@ -192,7 +212,7 @@ void DoMovie(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void MovieLoadState(const char * filename) {
+void MovieLoadState(void) {
 
 
 	if (Movie.ReadOnly == 1 && Movie.Status == Playback)  {
@@ -245,6 +265,12 @@ static int MovieGetSize(FILE* fp) {
 
 	fpos = ftell(fp);//save current pos
 
+   if (fpos < 0)
+   {
+      YabSetError(YAB_ERR_OTHER, "MovieGetSize fpos is negative");
+      return 0;
+   }
+
 	fseek (fp,0,SEEK_END);
 	size=ftell(fp);
 
@@ -369,10 +395,10 @@ void SaveMovieInState(FILE* fp, IOCheck_struct check) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void MovieReadState(FILE* fp, const char * filename) {
+void MovieReadState(FILE* fp) {
 
 	ReadMovieInState(fp);
-	MovieLoadState(filename);//file pointer and truncation
+	MovieLoadState();//file pointer and truncation
 
 }
 
@@ -380,17 +406,25 @@ void ReadMovieInState(FILE* fp) {
 
 	struct MovieBufferStruct tempbuffer;
 	int fpos;
+   size_t num_read = 0;
 
 	//overwrite the main movie on disk if we are recording or read+write playback
 	if(Movie.Status == Recording || (Movie.Status == Playback && Movie.ReadOnly == 0)) {
 
 		fpos=ftell(fp);//where we are in the savestate
-		fread(&tempbuffer.size, 4, 1, fp);//size
+
+      if (fpos < 0)
+      {
+         YabSetError(YAB_ERR_OTHER, "ReadMovieInState fpos is negative");
+         return;
+      }
+
+      num_read = fread(&tempbuffer.size, 4, 1, fp);//size
 		if ((tempbuffer.data = (char *)malloc(tempbuffer.size)) == NULL)
 		{
 			return;
 		}
-		fread(tempbuffer.data, 1, tempbuffer.size, fp);//movie
+      num_read = fread(tempbuffer.data, 1, tempbuffer.size, fp);//movie
 		fseek(fp, fpos, SEEK_SET);//reset savestate position
 
 		rewind(Movie.fp);
@@ -404,16 +438,23 @@ void ReadMovieInState(FILE* fp) {
 struct MovieBufferStruct ReadMovieIntoABuffer(FILE* fp) {
 
 	int fpos;
-	struct MovieBufferStruct tempbuffer;
+   struct MovieBufferStruct tempbuffer = { 0 };
+   size_t num_read = 0;
 
 	fpos = ftell(fp);//save current pos
 
+   if (fpos < 0)
+   {
+      YabSetError(YAB_ERR_OTHER, "ReadMovieIntoABuffer fpos is negative");
+      return tempbuffer;
+   }
+
 	fseek (fp,0,SEEK_END);
 	tempbuffer.size=ftell(fp);  //get size
 	rewind(fp);
 
 	tempbuffer.data = (char*) malloc (sizeof(char)*tempbuffer.size);
-	fread (tempbuffer.data, 1, tempbuffer.size, fp);
+   num_read = fread(tempbuffer.data, 1, tempbuffer.size, fp);
 
 	fseek(fp, fpos, SEEK_SET); //reset back to correct pos
 	return(tempbuffer);
@@ -425,7 +466,7 @@ const char *MakeMovieStateName(const char *filename) {
 
 	static char *retbuf = NULL;  // Save the pointer to avoid memory leaks
 	if(Movie.Status == Recording || Movie.Status == Playback) {
-		const unsigned long newsize = strlen(filename) + 5 + 1;
+		const size_t newsize = strlen(filename) + 5 + 1;
 		free(retbuf);
 		retbuf = malloc(newsize);
 		if (!retbuf) {
@@ -447,6 +488,10 @@ void TestWrite(struct MovieBufferStruct tempbuffer) {
 	FILE* tempbuffertest;
 
 	tempbuffertest=fopen("rmiab.txt", "wb");
+
+   if (!tempbuffertest)
+      return;
+
 	fwrite (tempbuffer.data, 1, tempbuffer.size, tempbuffertest);
 	fclose(tempbuffertest);
 }
