@@ -88,7 +88,7 @@ static void NetlinkMSRChange(u8 set, u8 clear)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL NetlinkReadByte(u32 addr)
+u8 FASTCALL NetlinkReadByte(SH2_struct *sh, u32 addr)
 {
    u8 ret;
 
@@ -227,7 +227,7 @@ void remove_all_chars(char* str, char c)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL NetlinkWriteByte(u32 addr, u8 val)
+void FASTCALL NetlinkWriteByte(SH2_struct *sh, u32 addr, u8 val)
 {
    switch (addr & 0xFFFFF)
    {
@@ -618,7 +618,7 @@ void FASTCALL NetlinkWriteByte(u32 addr, u8 val)
 
 //////////////////////////////////////////////////////////////////////////////
 
-int NetlinkInit(const char *setting)
+int NetlinkInit(const char *ip, const char *port)
 {  
    if ((NetlinkArea = malloc(sizeof(Netlink))) == NULL)
    {
@@ -656,30 +656,24 @@ int NetlinkInit(const char *setting)
    NetlinkArea->reg.SREG[7] = 50;
    NetlinkArea->connect_timeout = NetlinkArea->reg.SREG[7] * 1000000;
 
-   if (setting == NULL || strcmp(setting, "") == 0)
-   {
-      // Use Loopback ip and port 1337
+   if (ip == NULL || strcmp(ip, "") == 0)
+      // Use Loopback ip
       sprintf(NetlinkArea->ipstring, "127.0.0.1");
-      sprintf(NetlinkArea->portstring, "31337");
-   }
+	else
+		strcpy(NetlinkArea->ipstring, ip);
+
+	if (port == NULL || strcmp(port, "") == 0)
+		// Default port
+      sprintf(NetlinkArea->portstring, "1337");
    else
    {
-      char *p;
-      p = strchr(setting, '\n');
-      if (p == NULL)
+      size_t port_len = strlen(port);
+      if (port_len >= 6)
       {
-         strcpy(NetlinkArea->ipstring, setting);
-         sprintf(NetlinkArea->portstring, "1337");
-      }
-      else
-      {
-         memcpy(NetlinkArea->ipstring, setting, (int)(p - setting));
-         NetlinkArea->ipstring[(p - setting)] = '\0';
-         if (strlen(p+1) == 0)
-            sprintf(NetlinkArea->portstring, "1337");
-         else
-            strcpy(NetlinkArea->portstring, p+1);
+         YabSetError(YAB_ERR_OTHER, "Netlink port is too long");
+         return 0;
       }
+      strcpy(NetlinkArea->portstring, port);
    }
 
 #ifdef USESOCKET
