@@ -61,7 +61,11 @@ int Check_Skip_Key();
 
 PerInterface_struct PERDIRECTX = {
 PERCORE_DIRECTX,
+#ifdef HAVE_XINPUT
+"DirectX/XInput Input Interface",
+#else
 "DirectX Input Interface",
+#endif
 PERDXInit,
 PERDXDeInit,
 PERDXHandleEvents,
@@ -118,10 +122,10 @@ BOOL bCleanupCOM=FALSE;
 
 HRESULT SetupForIsXInputDevice()
 {
-	IWbemLocator *pIWbemLocator = NULL;
-	IEnumWbemClassObject *pEnumDevices = NULL;
-	IWbemClassObject *pDevices[20] = {0};
-	IWbemServices *pIWbemServices = NULL;
+IWbemLocator * pIWbemLocator = NULL;
+	IEnumWbemClassObject * pEnumDevices = NULL;
+	IWbemClassObject * pDevices[20] = {0};
+	IWbemServices * pIWbemServices = NULL;
 	BSTR bstrNamespace = NULL;
 	BSTR bstrDeviceID = NULL;
 	BSTR bstrClassName = NULL;
@@ -130,57 +134,54 @@ HRESULT SetupForIsXInputDevice()
 	UINT iDevice = 0;
 	VARIANT var;
 	HRESULT hr;
-
-	hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
+	
+	hr = CoInitialize(NULL);
 	bCleanupCOM = SUCCEEDED(hr);
 	
-	// Create WMI
-	hr = CoCreateInstance( &CLSID_WbemContext,
-		NULL,
-		CLSCTX_INPROC_SERVER,
-		&IID_IWbemContext,
-		(LPVOID*) &pIWbemLocator);
-	if( FAILED(hr) || pIWbemLocator == NULL )
+	hr = CoCreateInstance(&CLSID_WbemLocator,
+	                      NULL,
+	                      CLSCTX_INPROC_SERVER,
+	                      &IID_IWbemLocator,
+	                      (LPVOID *) &pIWbemLocator);
+	if (FAILED(hr) || pIWbemLocator == NULL) {
+		YabSetError(YAB_ERR_CANNOTINIT, "pIWbemLocator");
 		goto bail;
-
-	bstrNamespace = SysAllocString( L"\\\\.\\root\\cimv2" );if( bstrNamespace == NULL ) goto bail;
-	bstrClassName = SysAllocString( L"Win32_PNPEntity" );   if( bstrClassName == NULL ) goto bail;
-	bstrDeviceID  = SysAllocString( L"DeviceID" );          if( bstrDeviceID == NULL )  goto bail;
-
-	// Connect to WMI 	
-	hr = IWbemLocator_ConnectServer(pIWbemLocator, bstrNamespace, NULL, NULL, NULL, 
-		0L, NULL, NULL, &pIWbemServices );
-
-	if( FAILED(hr) || pIWbemServices == NULL )
+	}
+	
+	bstrNamespace = SysAllocString(L"\\\\.\\root\\cimv2"); if (bstrNamespace == NULL) {goto bail;}
+	bstrClassName = SysAllocString(L"Win32_PNPEntity");    if (bstrClassName == NULL) {goto bail;}
+	bstrDeviceID  = SysAllocString(L"DeviceID");           if (bstrDeviceID == NULL)  {goto bail;}
+	
+	hr = IWbemLocator_ConnectServer(pIWbemLocator, bstrNamespace, NULL, NULL, 0L,
+	                                0L, NULL, NULL, &pIWbemServices);
+	if (FAILED(hr) || pIWbemServices == NULL) 
+   {
+		YabSetError(YAB_ERR_CANNOTINIT, "pIWbemServices");
 		goto bail;
-
-	// Switch security level to IMPERSONATE
-	CoSetProxyBlanket( (IUnknown *)pIWbemServices, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE, NULL, 
-		RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE );                    
-
-	hr = IWbemServices_CreateInstanceEnum( pIWbemServices, bstrClassName, 0, NULL, &pEnumDevices ); 
-	if( FAILED(hr) || pEnumDevices == NULL )
+	}
+	
+	CoSetProxyBlanket((IUnknown *) pIWbemServices, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE, NULL,
+	                  RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE);
+	
+	hr = IWbemServices_CreateInstanceEnum(pIWbemServices, bstrClassName, 0, NULL, &pEnumDevices);
+	if (FAILED(hr) || pEnumDevices == NULL) 
+   {
+		YabSetError(YAB_ERR_CANNOTINIT, "pEnumDevices");
 		goto bail;
-
-	// Loop over all devices
-	for(;;)
-	{
-		// Get 20 at a time		
-		hr = IEnumWbemClassObject_Next( pEnumDevices, 10000, 20, pDevices, &uReturned );
+	}
+	
+	for (;;) 
+   {
+		// Get 20 at a time
+		hr = IEnumWbemClassObject_Next(pEnumDevices, 10000, 20, pDevices, &uReturned);
 		if( FAILED(hr) || 
 			uReturned == 0 )
 			break;
-
-		for( iDevice=0; iDevice<uReturned; iDevice++ )
-		{
-			// For each device, get its device ID
-			hr = IWbemClassObject_Get( pDevices[iDevice], bstrDeviceID, 0L, &var, NULL, NULL );
-			if( SUCCEEDED( hr ) && var.vt == VT_BSTR && var.bstrVal != NULL )
-			{
-				// Check if the device ID contains "IG_".  If it does, then it's an XInput device
-				// This information can not be found from DirectInput 
-				if( wcsstr( var.bstrVal, L"IG_" ) )
-				{
+		for (iDevice = 0; iDevice < uReturned; iDevice++) 
+      {
+			hr = IWbemClassObject_Get(pDevices[iDevice], bstrDeviceID, 0L, &var, NULL, NULL);
+			if (SUCCEEDED(hr) && var.vt == VT_BSTR && var.bstrVal != NULL) {
+				if (wcsstr(var.bstrVal, L"IG_")) {
 					// If it does, then get the VID/PID from var.bstrVal
 					DWORD dwPid = 0, dwVid = 0;
 					WCHAR* strVid;
@@ -239,6 +240,7 @@ void CleanupForIsXInputDevice()
 		pNode = (XINPUT_DEVICE_NODE *)pNode->pNext;
 		SAFE_DELETE( pDelete );
 	}
+   g_pXInputDeviceList = NULL;
 
 	if( bCleanupCOM )
 		CoUninitialize();
@@ -266,7 +268,8 @@ BOOL IsXInputDevice( const GUID* guidProduct )
 BOOL CALLBACK EnumPeripheralsCallback (LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef)
 {
    if (GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_GAMEPAD ||
-       GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_JOYSTICK)
+       GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_JOYSTICK ||
+       GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_1STPERSON)//xbox one controller uses this type
    {     
 #ifdef HAVE_XINPUT
 		if (IsXInputDevice(&lpddi->guidProduct))
@@ -294,7 +297,7 @@ BOOL CALLBACK EnumPeripheralsCallback (LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef)
 
 int PERDXInit(void)
 {
-	char tempstr[512];
+	char tempstr[512];//sprintf(tempstr, "Input. DirectInput8Create error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
 	HRESULT ret;
 	int user_index=0;
 	u32 i;
@@ -337,7 +340,7 @@ int PERDXInit(void)
 				return -1;
 		}
 	}
-	PerPortReset();
+
 	//LoadDefaultPort1A();
 	PERCORE_INITIALIZED = 1;
 	return 0;
@@ -371,25 +374,46 @@ void PERDXDeInit(void)
 
 void PollAxisAsButton(u32 pad, int min_id, int max_id, int deadzone, int val)
 {
+    //deadzone is part of the id
 	if ( val < -deadzone )
 	{
-		DX_PerKeyUp( pad, 0, max_id );
-		DX_PerKeyDown( pad, 0, min_id );
+		DX_PerKeyUp( pad, deadzone, max_id );
+		DX_PerKeyDown( pad, deadzone, min_id );
 	}
 	else if ( val > deadzone )
 	{
-		DX_PerKeyUp( pad, 0, min_id );
-		DX_PerKeyDown( pad, 0, max_id );
+		DX_PerKeyUp( pad, deadzone, min_id );
+		DX_PerKeyDown( pad, deadzone, max_id );
 	}
 	else
 	{
-		DX_PerKeyUp( pad, 0, min_id );
-		DX_PerKeyUp( pad, 0, max_id );
+		DX_PerKeyUp( pad, deadzone, min_id );
+		DX_PerKeyUp( pad, deadzone, max_id );
 	}
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
+void PollTriggerAsButton(u32 pad, int min_id, int max_id, int deadzone, int val)
+{
+#ifdef HAVE_XINPUT
+   //stick value is set to this PollKeys
+   u32 stick = XINPUT_GAMEPAD_TRIGGER_THRESHOLD;
+   if (val > deadzone)
+   {
+      DX_PerKeyUp(pad, stick, max_id);
+      DX_PerKeyDown(pad, stick, min_id);
+   }
+   else
+   {
+      DX_PerKeyUp(pad, stick, min_id);
+      DX_PerKeyUp(pad, stick, max_id);
+   }
+#endif
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 #ifdef HAVE_XINPUT
 void PollXInputButtons(u32 pad, XINPUT_STATE *state)
 {
@@ -425,31 +449,40 @@ void PollKeys(void)
 		if (dev_list[i].is_xinput_device)
 		{
 			XINPUT_STATE state;
+         u8 axislx, axisly, axisrx, axisry;
 			ZeroMemory( &state, sizeof(XINPUT_STATE) );
-			if (XInputGetState(dev_list[i].user_index, &state) != ERROR_DEVICE_NOT_CONNECTED)
+			if (XInputGetState(dev_list[i].user_index, &state) == ERROR_DEVICE_NOT_CONNECTED)
 				continue;
 
+         //convert signed values to 0-255
+         axislx = ((state.Gamepad.sThumbLX >> 8) - 0x80) & 0xff;
+         axisly = ~(((state.Gamepad.sThumbLY >> 8) - 0x80) & 0xff);//invert so that up == 0x00
+         axisrx = ((state.Gamepad.sThumbRX >> 8) - 0x80) & 0xff;
+         axisry = ~(((state.Gamepad.sThumbRY >> 8) - 0x80) & 0xff);
+
 			// Handle axis			
-			DX_PerAxisValue(i, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XI_THUMBLX, (u8)((state.Gamepad.sThumbLX) >> 8));
-			DX_PerAxisValue(i, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XI_THUMBLY, (u8)((state.Gamepad.sThumbLY) >> 8));
-			DX_PerAxisValue(i, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XI_THUMBRX, (u8)((state.Gamepad.sThumbRX) >> 8));
-			DX_PerAxisValue(i, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XI_THUMBRY, (u8)((state.Gamepad.sThumbRY) >> 8));
+			DX_PerAxisValue(i, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XI_THUMBLX + PAD_DIR_AXISLEFT, axislx);
+			DX_PerAxisValue(i, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XI_THUMBLY + PAD_DIR_AXISDOWN, axisly);
+			DX_PerAxisValue(i, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XI_THUMBRX + PAD_DIR_AXISLEFT, axisrx);
+			DX_PerAxisValue(i, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XI_THUMBRY + PAD_DIR_AXISDOWN, axisry);
 			DX_PerAxisValue(i, XINPUT_GAMEPAD_TRIGGER_THRESHOLD, XI_TRIGGERL, state.Gamepad.bLeftTrigger);
 			DX_PerAxisValue(i, XINPUT_GAMEPAD_TRIGGER_THRESHOLD, XI_TRIGGERR, state.Gamepad.bRightTrigger);
 			
 			// Left Stick
-			PollAxisAsButton(i, XI_THUMBL+PAD_DIR_AXISLEFT, XI_THUMBL+PAD_DIR_AXISRIGHT,
+			PollAxisAsButton(i, XI_THUMBLX+PAD_DIR_AXISLEFT, XI_THUMBLX+PAD_DIR_AXISRIGHT,
 								XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, state.Gamepad.sThumbLX);
-			PollAxisAsButton(i, XI_THUMBL+PAD_DIR_AXISUP, XI_THUMBL+PAD_DIR_AXISDOWN,
+			PollAxisAsButton(i, XI_THUMBLY+PAD_DIR_AXISUP, XI_THUMBLY+PAD_DIR_AXISDOWN,
 								XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, state.Gamepad.sThumbLY);
 
 			// Right Stick
-			PollAxisAsButton(i, XI_THUMBR+PAD_DIR_AXISLEFT, XI_THUMBR+PAD_DIR_AXISRIGHT,
+			PollAxisAsButton(i, XI_THUMBRX+PAD_DIR_AXISLEFT, XI_THUMBRX+PAD_DIR_AXISRIGHT,
 								XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, state.Gamepad.sThumbRX);
-			PollAxisAsButton(i, XI_THUMBR+PAD_DIR_AXISUP, XI_THUMBR+PAD_DIR_AXISDOWN,
+			PollAxisAsButton(i, XI_THUMBRY+PAD_DIR_AXISUP, XI_THUMBRY+PAD_DIR_AXISDOWN,
 								XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, state.Gamepad.sThumbRY);
 
 			PollXInputButtons(i, &state);
+         PollTriggerAsButton(i, XI_TRIGGERL, XI_TRIGGERL, XINPUT_GAMEPAD_TRIGGER_THRESHOLD, state.Gamepad.bLeftTrigger);
+         PollTriggerAsButton(i, XI_TRIGGERR, XI_TRIGGERR, XINPUT_GAMEPAD_TRIGGER_THRESHOLD, state.Gamepad.bRightTrigger);
 			continue;
 		}
 		else if (dev_list[i].lpDIDevice == NULL)
@@ -620,32 +653,34 @@ u32 PERDXScan(u32 flags)
 		{
 			XINPUT_STATE state;
 			ZeroMemory( &state, sizeof(XINPUT_STATE) );
-			if (XInputGetState(dev_list[i].user_index, &state) != ERROR_DEVICE_NOT_CONNECTED)
+			if (XInputGetState(dev_list[i].user_index, &state) == ERROR_DEVICE_NOT_CONNECTED)
 				continue;
 
+            //min_id max_id have to match for scan and poll
 			// Handle axis		
 			if (flags & PERSF_AXIS)
 			{
 				// L Thumb
 				if ((scan = ScanXInputAxis(i, state.Gamepad.sThumbLX,
 													XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE,
-													XI_THUMBLX, XI_THUMBLX)) != 0)
+               XI_THUMBLX + PAD_DIR_AXISLEFT, XI_THUMBLX + PAD_DIR_AXISRIGHT)) != 0)
 					return scan;
 				if ((scan = ScanXInputAxis(i, state.Gamepad.sThumbLY,
 													XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE,
-													XI_THUMBLY, XI_THUMBLY)) != 0)
+               XI_THUMBLY + PAD_DIR_AXISUP, XI_THUMBLY + PAD_DIR_AXISDOWN)) != 0)
 					return scan;
 
 				// R Thumb
 				if ((scan = ScanXInputAxis(i, state.Gamepad.sThumbRX,
 													XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE,
-													XI_THUMBRX, XI_THUMBRX)) != 0)
-					return scan;
-				if ((scan = ScanXInputAxis(i, state.Gamepad.sThumbRY,
-													XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE,
-													XI_THUMBRY, XI_THUMBRY)) != 0)
+               XI_THUMBRX + PAD_DIR_AXISLEFT, XI_THUMBRX + PAD_DIR_AXISRIGHT)) != 0)
 					return scan;
 
+				if ((scan = ScanXInputAxis(i, state.Gamepad.sThumbRY,
+													XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE,
+               XI_THUMBRY + PAD_DIR_AXISUP, XI_THUMBRY + PAD_DIR_AXISDOWN)) != 0)
+					return scan;
+            
 				// L Trigger
 				if ((scan = ScanXInputTrigger(i, state.Gamepad.bLeftTrigger, XINPUT_GAMEPAD_TRIGGER_THRESHOLD, 
 													XINPUT_GAMEPAD_TRIGGER_THRESHOLD, XI_TRIGGERL)) != 0)
