@@ -36,6 +36,8 @@ YabauseThread::YabauseThread( QObject* o )
 	mPause = true;
 	mTimerId = -1;
 	mInit = -1;
+	memset(&mYabauseConf, 0, sizeof(mYabauseConf));
+	showFPS = false;
 }
 
 YabauseThread::~YabauseThread()
@@ -166,8 +168,56 @@ void YabauseThread::reloadControllers()
 					break;
 				}
 				case PERWHEEL:
-					QtYabause::mainWindow()->appendLog( "Wheel controller type is not yet supported" );
-					break;
+            {
+               PerAnalog_struct* analogbits = PerWheelAdd(port == 1 ? &PORTDATA1 : &PORTDATA2);
+
+               settings->beginGroup(QString("Input/Port/%1/Id/%2/Controller/%3/Key").arg(port).arg(id).arg(type));
+               QStringList analogKeys = settings->childKeys();
+               settings->endGroup();
+
+               analogKeys.sort();
+               foreach(const QString& analogKey, analogKeys)
+               {
+                  const QString key = settings->value(QString(UIPortManager::mSettingsKey).arg(port).arg(id).arg(type).arg(analogKey)).toString();
+
+                  PerSetKey(key.toUInt(), analogKey.toUInt(), analogbits);
+               }
+               break;
+            }
+            case PERMISSIONSTICK:
+            {
+               PerAnalog_struct* analogbits = PerMissionStickAdd(port == 1 ? &PORTDATA1 : &PORTDATA2);
+
+               settings->beginGroup(QString("Input/Port/%1/Id/%2/Controller/%3/Key").arg(port).arg(id).arg(type));
+               QStringList analogKeys = settings->childKeys();
+               settings->endGroup();
+
+               analogKeys.sort();
+               foreach(const QString& analogKey, analogKeys)
+               {
+                  const QString key = settings->value(QString(UIPortManager::mSettingsKey).arg(port).arg(id).arg(type).arg(analogKey)).toString();
+
+                  PerSetKey(key.toUInt(), analogKey.toUInt(), analogbits);
+               }
+               break;
+            }
+            case PERTWINSTICKS:
+            {
+               PerAnalog_struct* analogbits = PerTwinSticksAdd(port == 1 ? &PORTDATA1 : &PORTDATA2);
+
+               settings->beginGroup(QString("Input/Port/%1/Id/%2/Controller/%3/Key").arg(port).arg(id).arg(type));
+               QStringList analogKeys = settings->childKeys();
+               settings->endGroup();
+
+               analogKeys.sort();
+               foreach(const QString& analogKey, analogKeys)
+               {
+                  const QString key = settings->value(QString(UIPortManager::mSettingsKey).arg(port).arg(id).arg(type).arg(analogKey)).toString();
+
+                  PerSetKey(key.toUInt(), analogKey.toUInt(), analogbits);
+               }
+               break;
+            }
 				case PER3DPAD:
 				{
 					PerAnalog_struct* analogbits = Per3DPadAdd( port == 1 ? &PORTDATA1 : &PORTDATA2 );
@@ -284,8 +334,13 @@ void YabauseThread::reloadSettings()
 	resetYabauseConf();
 
 	// read & apply settings
-	mYabauseConf.m68kcoretype = vs->value( "Advanced/M68KCore", mYabauseConf.m68kcoretype ).toInt();
+   mYabauseConf.m68kcoretype = vs->value("Advanced/68kCore", mYabauseConf.m68kcoretype).toInt();
 	mYabauseConf.percoretype = vs->value( "Input/PerCore", mYabauseConf.percoretype ).toInt();
+	mYabauseConf.sh1coretype = vs->value( "Advanced/SH1Interpreter", mYabauseConf.sh1coretype ).toInt();
+	mYabauseConf.use_cd_block_lle = vs->value( "Advanced/EnableCDBlockLLE", mYabauseConf.use_cd_block_lle ).toBool();
+   mYabauseConf.use_sh2_dma_timing = vs->value("Advanced/EnableSh2DmaTiming", mYabauseConf.use_sh2_dma_timing).toBool();
+   mYabauseConf.use_scu_dma_timing = vs->value("Advanced/EnableScuDmaTiming", mYabauseConf.use_scu_dma_timing).toBool();
+   mYabauseConf.sh2_cache_enabled = vs->value("Advanced/EnableSh2Cache", mYabauseConf.sh2_cache_enabled).toBool();
 	mYabauseConf.sh2coretype = vs->value( "Advanced/SH2Interpreter", mYabauseConf.sh2coretype ).toInt();
 	mYabauseConf.vidcoretype = vs->value( "Video/VideoCore", mYabauseConf.vidcoretype ).toInt();
 	mYabauseConf.osdcoretype = vs->value( "Video/OSDCore", mYabauseConf.osdcoretype ).toInt();
@@ -309,13 +364,24 @@ void YabauseThread::reloadSettings()
 			case 'L': mYabauseConf.regionid = 0xD; break;
 		}
 	}
-	mYabauseConf.biospath = strdup( vs->value( "General/Bios", mYabauseConf.biospath ).toString().toLatin1().constData() );
-	mYabauseConf.cdpath = strdup( vs->value( "General/CdRomISO", mYabauseConf.cdpath ).toString().toLatin1().constData() ); 
+	if (vs->value("General/EnableEmulatedBios", false).toBool())
+		mYabauseConf.biospath = strdup( "" );
+	else
+		mYabauseConf.biospath = strdup( vs->value( "General/Bios", mYabauseConf.biospath ).toString().toLatin1().constData() );
+	mYabauseConf.cdpath = strdup( vs->value( "General/CdRomISO", mYabauseConf.cdpath ).toString().toLatin1().constData() );
+   mYabauseConf.ssfpath = strdup(vs->value("General/SSFPath", mYabauseConf.ssfpath).toString().toLatin1().constData());
+   mYabauseConf.play_ssf = vs->value("General/PlaySSF", false).toBool();
    showFPS = vs->value( "General/ShowFPS", false ).toBool();
+	mYabauseConf.usethreads = (int)vs->value( "General/EnableMultiThreading", mYabauseConf.usethreads ).toBool();
+	mYabauseConf.numthreads = vs->value( "General/NumThreads", mYabauseConf.numthreads ).toInt();
 	mYabauseConf.buppath = strdup( vs->value( "Memory/Path", mYabauseConf.buppath ).toString().toLatin1().constData() );
+	mYabauseConf.sh1rompath = strdup( vs->value( "SH1ROM/Path", mYabauseConf.mpegpath ).toString().toLatin1().constData() );
 	mYabauseConf.mpegpath = strdup( vs->value( "MpegROM/Path", mYabauseConf.mpegpath ).toString().toLatin1().constData() );
 	mYabauseConf.cartpath = strdup( vs->value( "Cartridge/Path", mYabauseConf.cartpath ).toString().toLatin1().constData() );
+	mYabauseConf.modemip = strdup( vs->value( "Cartridge/ModemIP", mYabauseConf.modemip ).toString().toLatin1().constData() );
+	mYabauseConf.modemport = strdup( vs->value( "Cartridge/ModemPort", mYabauseConf.modemport ).toString().toLatin1().constData() );
 	mYabauseConf.videoformattype = vs->value( "Video/VideoFormat", mYabauseConf.videoformattype ).toInt();
+   mYabauseConf.use_new_scsp = (int)vs->value("Sound/NewScsp", mYabauseConf.use_new_scsp).toBool();
 	
 	emit requestSize( QSize( vs->value( "Video/WinWidth", 0 ).toInt(), vs->value( "Video/WinHeight", 0 ).toInt() ) );
 	emit requestFullscreen( vs->value( "Video/Fullscreen", false ).toBool() );
@@ -344,6 +410,8 @@ void YabauseThread::resetYabauseConf()
 	// fill default structure
 	mYabauseConf.m68kcoretype = M68KCORE_C68K;
 	mYabauseConf.percoretype = QtYabause::defaultPERCore().id;
+	mYabauseConf.sh1coretype = SH2CORE_DEFAULT;
+	mYabauseConf.use_cd_block_lle = 0;
 	mYabauseConf.sh2coretype = SH2CORE_DEFAULT;
 	mYabauseConf.vidcoretype = QtYabause::defaultVIDCore().id;
 	mYabauseConf.sndcoretype = QtYabause::defaultSNDCore().id;
@@ -356,6 +424,10 @@ void YabauseThread::resetYabauseConf()
 	mYabauseConf.mpegpath = 0;
 	mYabauseConf.cartpath = 0;
 	mYabauseConf.videoformattype = VIDEOFORMATTYPE_NTSC;
+	mYabauseConf.skip_load = 0;
+	int numThreads = QThread::idealThreadCount();	
+	mYabauseConf.usethreads = numThreads <= 1 ? 0 : 1;
+	mYabauseConf.numthreads = numThreads < 0 ? 1 : numThreads;
 }
 
 void YabauseThread::timerEvent( QTimerEvent* )
