@@ -95,7 +95,7 @@
     </message>
     <message>
         <location filename="../ui/UIBackupRam.ui" line="70"/>
-        <source>Save Informations</source>
+        <source>Save Information</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
