@@ -21,7 +21,7 @@
 #include "../CommonDialogs.h"
 #include "UIYabause.h"
 
-int M68KDis(u32 addr, char *string)
+int M68KDis(void *context, u32 addr, char *string)
 {
    return (int)(M68KDisasm(addr, string) - addr);
 }
@@ -34,7 +34,7 @@ void M68KBreakpointHandler (u32 addr)
 }
 
 UIDebugM68K::UIDebugM68K( YabauseThread *mYabauseThread, QWidget* p )
-	: UIDebugCPU(mYabauseThread, p )
+   : UIDebugCPU(PROC_M68K, mYabauseThread, p )
 {
    this->setWindowTitle(QtYabause::translate("Debug M68K"));
    gbRegisters->setTitle(QtYabause::translate("M68K Registers"));
@@ -146,7 +146,9 @@ u32 UIDebugM68K::getRegister(int index, int *size)
       case 17:
          value = m68kregs.PC;
          break;
-      default: break;
+      default: 
+			value = 0;
+			break;
    }
 
    *size = 4;
@@ -157,6 +159,9 @@ void UIDebugM68K::setRegister(int index, u32 value)
 {
    m68kregs_struct m68kregs;
 
+	memset(&m68kregs, 0, sizeof(m68kregs));
+	M68KGetRegisters(&m68kregs);
+
    switch (index)
    {
       case 0:
@@ -194,6 +199,8 @@ void UIDebugM68K::setRegister(int index, u32 value)
 
 bool UIDebugM68K::addCodeBreakpoint(u32 addr)
 {
+	if (!SoundRam)
+		return false;
    return M68KAddCodeBreakpoint(addr) == 0;     
 }
 
