@@ -20,7 +20,7 @@
 #include "../CommonDialogs.h"
 #include "UIYabause.h"
 
-int SCUDSPDis(u32 addr, char *string)
+int SCUDSPDis(void *context, u32 addr, char *string)
 {
    ScuDspDisasm((u8)addr, string);
    return 1;
@@ -34,7 +34,7 @@ void SCUDSPBreakpointHandler (u32 addr)
 }
 
 UIDebugSCUDSP::UIDebugSCUDSP( YabauseThread *mYabauseThread, QWidget* p )
-	: UIDebugCPU( mYabauseThread, p )
+   : UIDebugCPU( PROC_SCUDSP, mYabauseThread, p )
 {
    this->setWindowTitle(QtYabause::translate("Debug SCU DSP"));
    gbRegisters->setTitle(QtYabause::translate("DSP Registers"));
@@ -61,33 +61,30 @@ UIDebugSCUDSP::UIDebugSCUDSP( YabauseThread *mYabauseThread, QWidget* p )
    size.setWidth(lwRegisters->fontMetrics().averageCharWidth() * 80);
    lwDisassembledCode->setMinimumSize(size);
 
-   updateRegList();
-   if (ScuRegs)
-   {
-      scudspregs_struct regs;
-      const scucodebreakpoint_struct *cbp;
-      int i;
+	if (ScuRegs)
+	{
+		const scucodebreakpoint_struct *cbp;
+		int i;
 
-      cbp = ScuDspGetBreakpointList();
+		cbp = ScuDspGetBreakpointList();
 
-      for (i = 0; i < MAX_BREAKPOINTS; i++)
-      {
-         QString text;
-         if (cbp[i].addr != 0xFFFFFFFF)
-         {
-            text.sprintf("%08X", (int)cbp[i].addr);
-            lwCodeBreakpoints->addItem(text);
-         }
-      }
+		for (i = 0; i < MAX_BREAKPOINTS; i++)
+		{
+			QString text;
+			if (cbp[i].addr != 0xFFFFFFFF)
+			{
+				text.sprintf("%08X", (int)cbp[i].addr);
+				lwCodeBreakpoints->addItem(text);
+			}
+		}
 
-      lwDisassembledCode->setDisassembleFunction(SCUDSPDis);
-      lwDisassembledCode->setEndAddress(0x100);
-      lwDisassembledCode->setMinimumInstructionSize(1);
-      ScuDspGetRegisters(&regs);
-      updateCodeList(regs.PC);
+		lwDisassembledCode->setDisassembleFunction(SCUDSPDis);
+		lwDisassembledCode->setEndAddress(0x100);
+		lwDisassembledCode->setMinimumInstructionSize(1);
+		ScuDspSetBreakpointCallBack(SCUDSPBreakpointHandler);
+	}
 
-      ScuDspSetBreakpointCallBack(SCUDSPBreakpointHandler);
-   }
+	updateAll();
 }
 
 void UIDebugSCUDSP::updateRegList()
@@ -132,16 +129,16 @@ void UIDebugSCUDSP::updateRegList()
    str.sprintf("CT = %02X:%02X:%02X:%02X", regs.CT[0], regs.CT[1], regs.CT[2], regs.CT[3]);
    lwRegisters->addItem(str);
 
-   str.sprintf("RA =   %08lX", regs.RA0);
+   str.sprintf("RA =   %08X", regs.RA0);
    lwRegisters->addItem(str);
 
-   str.sprintf("WA =   %08lX", regs.WA0);
+   str.sprintf("WA =   %08X", regs.WA0);
    lwRegisters->addItem(str);
 
-   str.sprintf("RX =   %08lX", regs.RX);
+   str.sprintf("RX =   %08X", regs.RX);
    lwRegisters->addItem(str);
 
-   str.sprintf("RY =   %08lX", regs.RX);
+   str.sprintf("RY =   %08X", regs.RX);
    lwRegisters->addItem(str);
 
    str.sprintf("PH =       %04X", regs.P.part.H & 0xFFFF);
@@ -163,6 +160,17 @@ void UIDebugSCUDSP::updateCodeList(u32 addr)
    lwDisassembledCode->setPC(addr);
 }
 
+void UIDebugSCUDSP::updateAll()
+{
+	updateRegList();
+	if (ScuRegs)
+	{
+		scudspregs_struct regs;
+		ScuDspGetRegisters(&regs);
+		updateCodeList(regs.PC);
+	}
+}
+
 u32 UIDebugSCUDSP::getRegister(int index, int *size)
 {
    *size = 0;
@@ -175,22 +183,29 @@ void UIDebugSCUDSP::setRegister(int index, u32 value)
 
 bool UIDebugSCUDSP::addCodeBreakpoint(u32 addr)
 {
+	if (!ScuRegs)
+		return false;
    return ScuDspAddCodeBreakpoint(addr) == 0;     
 }
 
 bool UIDebugSCUDSP::delCodeBreakpoint(u32 addr)
 {
+	if (!ScuRegs)
+		return false;
    return ScuDspDelCodeBreakpoint(addr) == 0;
 }
 
 void UIDebugSCUDSP::stepInto()
 {
    ScuDspStep();
+   updateAll();
 }
 
 void UIDebugSCUDSP::reserved1()
 {
    const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for binary file" ), QtYabause::translate( "Binary Files (*.bin)" ) );
+	if (!ScuRegs)
+		return;
    if ( !s.isNull() )
       ScuDspSaveProgram(s.toLatin1());
 }
@@ -198,6 +213,8 @@ void UIDebugSCUDSP::reserved1()
 void UIDebugSCUDSP::reserved2()
 {
    const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for binary file" ), QtYabause::translate( "Binary Files (*.bin)" ) );
+	if (!ScuRegs)
+		return;
    if ( !s.isNull() )
       ScuDspSaveMD(s.toLatin1(), 0);
 }
@@ -205,6 +222,8 @@ void UIDebugSCUDSP::reserved2()
 void UIDebugSCUDSP::reserved3()
 {
    const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for binary file" ), QtYabause::translate( "Binary Files (*.bin)" ) );
+	if (!ScuRegs)
+		return;
    if ( !s.isNull() )
       ScuDspSaveMD(s.toLatin1(), 1);
 }
@@ -212,6 +231,8 @@ void UIDebugSCUDSP::reserved3()
 void UIDebugSCUDSP::reserved4()
 {
    const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for binary file" ), QtYabause::translate( "Binary Files (*.bin)" ) );
+	if (!ScuRegs)
+		return;
    if ( !s.isNull() )
       ScuDspSaveMD(s.toLatin1(), 2);
 }
@@ -219,6 +240,8 @@ void UIDebugSCUDSP::reserved4()
 void UIDebugSCUDSP::reserved5()
 {
    const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for binary file" ), QtYabause::translate( "Binary Files (*.bin)" ) );
+	if (!ScuRegs)
+		return;
    if ( !s.isNull() )
       ScuDspSaveMD(s.toLatin1(), 3);
 }
