@@ -24,22 +24,43 @@
 #include <QScrollBar>
 
 UIHexEditor::UIHexEditor( QWidget* p )
+{
+	proc = UIDebugCPU::PROC_MSH2;
+	rebuildTabs();
+}
+
+void UIHexEditor::rebuildTabs()
 {
    QList<QString> tabList;
    QList<u32> startList;
    QList<u32> endList;
-   tabList   << "All"      << "BIOS"     << "LWRAM"    << "HWRAM"     << 
-                "CS0"      << "CS1"      << "CS2"      << "68K RAM"   << 
-                "VDP1 RAM" << "VDP1 FB"  << "VDP2 RAM" << "VDP2 CRAM";
-   startList << 0x00000000 << 0x00000000 << 0x00200000 << 0x06000000 <<
-                0x02000000 << 0x04000000 << 0x05800000 << 0x05A00000 <<
-                0x05C00000 << 0x05C80000 << 0x05E00000 << 0x05F00000;
-   endList   << 0xFFFFFFFF << 0x0017FFFF << 0x002FFFFF << 0x060FFFFF <<
-                0x03FFFFFF << 0x04FFFFFF << 0x058FFFFF << 0x05AFFFFF <<
-                0x05C7FFFF << 0x05CFFFFF << 0x05EFFFFF << 0x05F7FFFF;
+
+   clear();
+   //if (proc == UIDebugCPU::PROC_MSH2 || proc == UIDebugCPU::PROC_SSH2)
+   if (proc != UIDebugCPU::PROC_SH1)
+   {
+      tabList   << "All"      << "BIOS"     << "LWRAM"    << "HWRAM"     << 
+         "CS0"      << "CS1"      << "CS2"      << "68K RAM"   << 
+         "VDP1 RAM" << "VDP1 FB"  << "VDP2 RAM" << "VDP2 CRAM";
+      startList << 0x00000000 << 0x00000000 << 0x00200000 << 0x06000000 <<
+         0x02000000 << 0x04000000 << 0x05800000 << 0x05A00000 <<
+         0x05C00000 << 0x05C80000 << 0x05E00000 << 0x05F00000;
+      endList   << 0x07FFFFFF << 0x0017FFFF << 0x002FFFFF << 0x060FFFFF <<
+         0x03FFFFFF << 0x04FFFFFF << 0x058FFFFF << 0x05AFFFFF <<
+         0x05C7FFFF << 0x05CFFFFF << 0x05EFFFFF << 0x05F7FFFF;
+   }
+   else
+   {
+      tabList   << "All" << "ROM" << "REG" << "DRAM" << "CS2" << "RAM Mirror";
+      startList << 0x00000000 << 0x00000000 << 0x05FFFE00 << 0x09000000 <<
+         0x0A000000 << 0x0F000000;
+      endList   << 0xFFFFFFFF << 0x0000FFFF << 0x05FFFFFF << 0x0907FFFF <<
+         0x0A1FFFFF << 0x0F07FFFF;
+   }
    for (int i=0; i < tabList.count(); i++)
    {
       UIHexEditorWnd *hexEditorWnd = new UIHexEditorWnd (this);
+      hexEditorWnd->setProc(proc);
       hexEditorWnd->setStartAddress(startList[i]);
       hexEditorWnd->setEndAddress(endList[i]);
       addTab(hexEditorWnd, tabList[i]);
@@ -65,14 +86,26 @@ u32 UIHexEditor::getEndAddress()
    return hexEditorWnd->getEndAddress();
 }
 
+void UIHexEditor::setProc( UIDebugCPU::PROCTYPE proc )
+{
+   this->proc = proc;
+   rebuildTabs();
+}
+
 bool UIHexEditor::saveSelected( QString filename )
 {
    UIHexEditorWnd *hexEditorWnd=(UIHexEditorWnd *)currentWidget();
    return hexEditorWnd->saveSelected(filename);
 }
 
+bool UIHexEditor::saveTab(QString filename)
+{
+   UIHexEditorWnd *hexEditorWnd = (UIHexEditorWnd *)currentWidget();
+   return hexEditorWnd->saveTab(filename);
+}
+
 UIHexEditorWnd::UIHexEditorWnd( QWidget* p )
-	: QAbstractScrollArea( p )
+   : QAbstractScrollArea( p )
 {
    gapSizeAddrHex = 10;
    gapSizeHexText = 16;
@@ -101,7 +134,6 @@ UIHexEditorWnd::UIHexEditorWnd( QWidget* p )
    setMouseTracking(true);
 }
 
-
 void UIHexEditorWnd::sliderUpdate(int value)
 {
    setCursorPos(cursorAddr);
@@ -140,6 +172,11 @@ u32 UIHexEditorWnd::getAddress()
    return cursorAddr;
 }
 
+void UIHexEditorWnd::setProc( UIDebugCPU::PROCTYPE proc )
+{
+   this->proc = proc;
+}
+
 void UIHexEditorWnd::setFont(const QFont &font)
 {
    QWidget::setFont(font);
@@ -160,27 +197,63 @@ void UIHexEditorWnd::goToAddress(u32 address, bool setCursor)
    viewport()->update();
 }
 
+u8 UIHexEditorWnd::readByte(u32 addr)
+{
+   if (proc == UIDebugCPU::PROC_MSH2 || proc == UIDebugCPU::PROC_SSH2)
+   {
+      if ((addr >= 0x05D00000 && addr < 0x05D80000) ||
+         (addr >= 0x05F80000 && addr < 0x05FC0000))
+         return MappedMemoryReadWordNocache(MSH2, addr & (~0x1)) >> ((1-(addr & 0x1))<<3);
+      else
+         return MappedMemoryReadByteNocache(MSH2, addr);
+   }
+   else if (proc == UIDebugCPU::PROC_SH1)
+      return SH1->MappedMemoryReadByte(SH1, addr);
+   else
+      return MappedMemoryReadByteNocache(MSH2, addr);
+}
+
+void UIHexEditorWnd::writeByte(u32 addr, u8 val)
+{
+   if (proc == UIDebugCPU::PROC_MSH2 || proc == UIDebugCPU::PROC_SSH2)
+   {
+      if ((addr >= 0x05D00000 && addr < 0x05D80000) ||
+         (addr >= 0x05F80000 && addr < 0x05FC0000))
+      {
+         u16 word = MappedMemoryReadWordNocache(MSH2, addr & (~0x1)) & (0xFF << ((addr & 0x1)<<3) );
+         word |= (val << ((1-(addr & 0x1))<<3));
+         MappedMemoryWriteWordNocache(MSH2, addr & (~0x1), word);
+      }
+      else
+         MappedMemoryWriteByteNocache(MSH2, addr, val);
+   }
+   else if (proc == UIDebugCPU::PROC_SH1)
+      SH1->MappedMemoryWriteByte(SH1, addr, val);
+   else
+      MappedMemoryWriteByteNocache(MSH2, addr, val);
+}
+
 void UIHexEditorWnd::clear(u32 index, int len)
 {
    for (int i=0; i < len; i++)
-      MappedMemoryWriteByte(index+i, 0);
+      writeByte(index+i, 0);
 }
 
 void UIHexEditorWnd::overwrite(s64 index, char ch)
 {
-   u8 data = MappedMemoryReadByte(index / 2);
+   u8 data = readByte(index / 2);
    char str[2] = { ch, '\0' };
    ch = strtol(str, NULL, 16);
    if (index % 2 == 0)
-      MappedMemoryWriteByte(index / 2, data & 0xF | (ch << 4));
+      writeByte(index / 2, data & 0xF | (ch << 4));
    else
-      MappedMemoryWriteByte(index / 2, data & 0xF0 | ch);
+      writeByte(index / 2, data & 0xF0 | ch);
    resetSelection();
 }
 
 void UIHexEditorWnd::overwrite(u32 addr, u8 data)
 {
-   MappedMemoryWriteByte(addr, data);
+   writeByte(addr, data);
    resetSelection();
 }
 
@@ -441,12 +514,12 @@ void UIHexEditorWnd::keyPressEdit(QKeyEvent *event, u64 posAddr)
       if (!textEdit)
       {
          for (u64 idx = getSelectionStart(); idx < getSelectionEnd(); idx++)
-            result.append(QString("%1").arg(MappedMemoryReadByte(idx), 2, 16, QChar('0')).toUpper());
+            result.append(QString("%1").arg(readByte(idx), 2, 16, QChar('0')).toUpper());
       }
       else
       {
          for (u64 idx = getSelectionStart(); idx < getSelectionEnd(); idx++)
-            result.append((char)MappedMemoryReadByte(idx));
+            result.append((char)readByte(idx));
       }
       QClipboard *clipboard = QApplication::clipboard();
       clipboard->setText(result);
@@ -719,7 +792,7 @@ void UIHexEditorWnd::drawHexArea(QPainter *painter, int firstLineIdx, u32 lastLi
             }
 
             // Paint hex value
-            text=QString("%1").arg(MappedMemoryReadByte(addr + lineIdx + colIdx - firstLineIdx), 2, 16, QChar('0')).toUpper();               
+            text=QString("%1").arg(readByte(addr + lineIdx + colIdx - firstLineIdx), 2, 16, QChar('0')).toUpper();               
             painter->drawText(xPos, yPos, text);
             xPos += text.length() * fontWidth;
          }
@@ -754,7 +827,7 @@ void UIHexEditorWnd::drawTextArea(QPainter *painter, int firstLineIdx, u32 lastL
                painter->setBackgroundMode(Qt::TransparentMode);
             }
 
-            QString text=QString((char)MappedMemoryReadByte(addr + lineIdx + colIdx));
+            QString text=QString((char)readByte(addr + lineIdx + colIdx));
 
             if (fontMetrics().width(text) == 0)
                text = QString('.');
@@ -969,30 +1042,41 @@ u64 UIHexEditorWnd::getSelectionEnd()
 
 bool UIHexEditorWnd::saveSelected(QString filename)
 {
-   FILE *fp=fopen(filename.toLatin1(), "wb");
-   u32 size=(u32)(getSelectionEnd()-getSelectionStart())/2;
-
-   if (fp == NULL)
-      return false;
-
-   u8 *buf = (unsigned char *)malloc(size);
-   if (buf == NULL)
-   {
-      fclose(fp);
-      return false;
-   }
-
-   for (u32 i = 0; i < size; i++)
-      buf[i] = MappedMemoryReadByte((getSelectionStart() / 2)+i);
-
-   if (fwrite((void *)buf, 1, size, fp) != size)
-   {
-      free(buf);
-      fclose(fp);
-      return false;
-   }
-
-   free(buf);
-   fclose(fp);
-   return true;
+	return saveMemory(filename, getSelectionStart(), getSelectionEnd());
 }
+
+bool UIHexEditorWnd::saveTab(QString filename)
+{
+	return saveMemory(filename, startAddress, endAddress + 1);
+}
+
+
+bool UIHexEditorWnd::saveMemory(QString filename, u32 startAddress, u32 endAddress)
+{
+	FILE *fp = fopen(filename.toLatin1(), "wb");
+	u32 size = (u32)(endAddress - startAddress);
+
+	if (fp == NULL)
+		return false;
+
+	u8 *buf = (unsigned char *)malloc(size);
+	if (buf == NULL)
+	{
+		fclose(fp);
+		return false;
+	}
+
+	for (u32 i = 0; i < size; i++)
+		buf[i] = readByte((startAddress)+i);
+
+	if (fwrite((void *)buf, 1, size, fp) != size)
+	{
+		free(buf);
+		fclose(fp);
+		return false;
+	}
+
+	free(buf);
+	fclose(fp);
+	return true;
+}
\ No newline at end of file
