@@ -25,6 +25,7 @@
 #include <QTabWidget>
 #include <QVBoxLayout>
 #include "../QtYabause.h"
+#include "UIDebugCPU.h"
 
 class UIHexEditorWnd : public QAbstractScrollArea
 {
@@ -38,10 +39,15 @@ public:
    void setEndAddress(u32 address);
    u32 getEndAddress();
    u32 getAddress();
+   void setProc( UIDebugCPU::PROCTYPE proc );
    void goToAddress(u32 address, bool setCursor=true);
    virtual void setFont(const QFont &font);
    bool saveSelected(QString filename);
+   bool saveTab(QString filename);
 private:
+   UIDebugCPU::PROCTYPE proc;
+   u8 readByte(u32 addr);
+   void writeByte(u32 addr, u8 val);
    void keyPressCursor(QKeyEvent *event);
    void keyPressSelect(QKeyEvent *event);
    void keyPressEdit(QKeyEvent *event, u64 posBa);
@@ -109,6 +115,7 @@ private slots:
    void autoScroll();
    void sliderUpdate(int value);
    virtual bool focusNextPrevChild(bool next);
+   bool saveMemory(QString filename, u32 startAddress, u32 endAddress);
 };
 
 class UIHexEditor : public QTabWidget
@@ -120,9 +127,12 @@ public:
    void goToAddress(u32 address, bool setCursor=true);
    u32 getStartAddress();
    u32 getEndAddress();
+   void setProc( UIDebugCPU::PROCTYPE proc );
    bool saveSelected(QString filename);
+   bool saveTab(QString filename);
 private:
-   ;
+   UIDebugCPU::PROCTYPE proc;
+   void rebuildTabs();
 };
 
 #endif // UIHEXEDITOR_H
