@@ -30,6 +30,9 @@ UIHexInput::UIHexInput( u32 value, int size, QWidget* p )
    QString text;
    sprintf(format, "%%0%dX", size * 2);
    text.sprintf(format, value);
+	
+	this->value = value;
+	this->size = size;
   
 	leValue->setValidator(new HexValidator(0x00000000, 0xFFFFFFFF >> (4-size * 8) , leValue));
    leValue->setText(text);
