@@ -31,6 +31,8 @@
 #include "UIDebugM68K.h"
 #include "UIDebugSCUDSP.h"
 #include "UIDebugSCSP.h"
+#include "UIDebugSCSPChan.h"
+#include "UIDebugSCSPDSP.h"
 #include "UIMemoryEditor.h"
 #include "UIMemoryTransfer.h"
 #include "UIAbout.h"
@@ -108,13 +110,21 @@ UIYabause::UIYabause( QWidget* parent )
 	mLogDock->setWidget( teLog );
 	addDockWidget( Qt::BottomDockWidgetArea, mLogDock );
 	mLogDock->setVisible( false );
+	mCanLog = true;
+	oldMouseX = oldMouseY = 0;
+	mouseCaptured = false;
+
+#ifndef SH2_TRACE
+	aTraceLogging->setVisible(false);
+#endif
+
 	// create emulator thread
 	mYabauseThread = new YabauseThread( this );
 	// create hide mouse timer
 	hideMouseTimer = new QTimer();
 	// create mouse cursor timer
 	mouseCursorTimer = new QTimer();
-	// connectionsdd
+	// connections
 	connect( mYabauseThread, SIGNAL( requestSize( const QSize& ) ), this, SLOT( sizeRequested( const QSize& ) ) );
 	connect( mYabauseThread, SIGNAL( requestFullscreen( bool ) ), this, SLOT( fullscreenRequested( bool ) ) );
 	connect( mYabauseThread, SIGNAL( requestVolumeChange( int ) ), this, SLOT( on_sVolume_valueChanged( int ) ) );
@@ -157,6 +167,13 @@ UIYabause::UIYabause( QWidget* parent )
 	mouseSensitivity = vs->value( "Input/GunMouseSensitivity", 100 ).toInt();
 	showMenuBarHeight = menubar->height();
 	translations = QtYabause::getTranslationList();
+	
+	VIDSoftSetBilinear(QtYabause::settings()->value( "Video/Bilinear", false ).toBool());
+}
+
+UIYabause::~UIYabause()
+{
+	mCanLog = false;
 }
 
 void UIYabause::showEvent( QShowEvent* e )
@@ -197,14 +214,38 @@ void UIYabause::closeEvent( QCloseEvent* e )
 }
 
 void UIYabause::keyPressEvent( QKeyEvent* e )
-{ PerKeyDown( e->key() ); }
+{ 
+	if (emulateMouse && mouseCaptured && e->key() == Qt::Key_Escape)
+		mouseCaptured = false;
+	else
+		PerKeyDown( e->key() ); 
+}
 
 void UIYabause::keyReleaseEvent( QKeyEvent* e )
 { PerKeyUp( e->key() ); }
 
+void UIYabause::leaveEvent( QEvent* e )
+{
+	if (emulateMouse && mouseCaptured)
+	{
+		// lock cursor to center
+		int midX = geometry().x()+(width()/2); // widget global x
+		int midY = geometry().y()+menubar->height()+toolBar->height()+(height()/2); // widget global y
+
+		QPoint newPos(midX, midY);
+		this->cursor().setPos(newPos);
+	}
+}
+
 void UIYabause::mousePressEvent( QMouseEvent* e )
 { 
-	PerKeyDown( (1 << 31) | e->button() );
+	if (emulateMouse && !mouseCaptured)
+	{
+		this->setCursor(Qt::BlankCursor);
+		mouseCaptured = true;
+	}
+	else
+		PerKeyDown( (1 << 31) | e->button() );
 }
 
 void UIYabause::mouseReleaseEvent( QMouseEvent* e )
@@ -226,30 +267,30 @@ void UIYabause::cursorRestore()
 
 void UIYabause::mouseMoveEvent( QMouseEvent* e )
 { 
-	int x = (e->x()-oldMouseX)*mouseXRatio;
-	int y = (oldMouseY-e->y())*mouseYRatio;
+	int midX = geometry().x()+(width()/2); // widget global x
+	int midY = geometry().y()+menubar->height()+toolBar->height()+(height()/2); // widget global y
+
+	int x = (e->x()-(width()/2))*mouseXRatio;
+	int y = ((menubar->height()+toolBar->height()+(height()/2))-e->y())*mouseYRatio;
 	int minAdj = mouseSensitivity/100;
 
 	// If minimum movement is less than x, wait until next pass to apply	
 	if (abs(x) < minAdj) x = 0;
 	if (abs(y) < minAdj) y = 0;
 
-	PerAxisMove((1 << 30), x, y);
-
-	oldMouseX = oldMouseX+(x/mouseXRatio);
-	oldMouseY = oldMouseY-(y/mouseYRatio);	
+	if (mouseCaptured)
+		PerAxisMove((1 << 30), x, y);
 
 	VolatileSettings* vs = QtYabause::volatileSettings();
 
 	if (!isFullScreen())
 	{
-		if (emulateMouse)
+		if (emulateMouse && mouseCaptured)
 		{
-			int menuToolHeight = menubar->height() + toolBar->height();
-			if (oldMouseY > menuToolHeight)
-				this->setCursor(Qt::BlankCursor);
-			else
-				this->setCursor(Qt::ArrowCursor);
+			// lock cursor to center
+			QPoint newPos(midX, midY);
+			this->cursor().setPos(newPos);
+			this->setCursor(Qt::BlankCursor);
 			return;
 		}
 		else
@@ -257,24 +298,17 @@ void UIYabause::mouseMoveEvent( QMouseEvent* e )
 	}
 	else
 	{
-		if (vs->value( "View/Menubar" ).toInt() == BD_SHOWONFSHOVER)
+		if (emulateMouse && mouseCaptured)
+		{
+			this->setCursor(Qt::BlankCursor);
+			return;
+		}
+		else if (vs->value( "View/Menubar" ).toInt() == BD_SHOWONFSHOVER)
 		{
 			if (e->y() < showMenuBarHeight)
 				menubar->show();
 			else
-			{
 				menubar->hide();
-				if (emulateMouse)
-				{
-					this->setCursor(Qt::BlankCursor);
-					return;
-				}
-			}
-		}
-		else if (emulateMouse)
-		{
-			this->setCursor(Qt::BlankCursor);
-			return;
 		}
 
 		hideMouseTimer->start(3 * 1000);
@@ -282,14 +316,87 @@ void UIYabause::mouseMoveEvent( QMouseEvent* e )
 	}
 }
 
+void UIYabause::resizeEvent( QResizeEvent* event )
+{
+	if (event->oldSize().width() != event->size().width())
+		fixAspectRatio(event->size().width());
+
+	QMainWindow::resizeEvent( event );
+}
+
+void UIYabause::adjustHeight(int & height)
+{
+   // Compensate for menubar and toolbar
+   VolatileSettings* vs = QtYabause::volatileSettings();
+   if (vs->value("View/Menubar").toInt() != BD_ALWAYSHIDE)
+      height += menubar->height();
+   if (vs->value("View/Toolbar").toInt() != BD_ALWAYSHIDE)
+      height += toolBar->height();
+}
+
+void UIYabause::resizeIntegerScaling()
+{
+   if (!VIDCore || VIDCore->id != VIDCORE_SOFT)
+      return;
+
+   if (isFullScreen() || emulateMouse)
+      return;
+
+   VolatileSettings* vs = QtYabause::volatileSettings();
+
+   if (!vs->value("Video/EnableIntegerPixelScaling").toBool())
+      return;
+
+   int multiplier = vs->value("Video/IntegerPixelScalingMultiplier").toInt();
+
+   int vdp2width = 0;
+   int vdp2height = 0;
+   int vdp2interlace = 0;
+
+   if (!VIDCore->GetNativeResolution)
+      return;
+
+   VIDCore->GetNativeResolution(&vdp2width, &vdp2height, &vdp2interlace);
+
+   if (vdp2width == 0 || vdp2height == 0)
+      return;
+
+   int width = 0;
+   int height = 0;
+
+   if (vdp2width < 640)
+      width = vdp2width * multiplier;
+   else
+      width = vdp2width * (multiplier / 2.0);
+
+   if (!vdp2interlace)
+      height = vdp2height * multiplier;
+   else
+      height = vdp2height * (multiplier / 2.0);
+
+   mYabauseGL->resize(width, height);
+
+   adjustHeight(height);
+
+   setMinimumSize(width, height);
+   resize(width, height);
+}
+
 void UIYabause::swapBuffers()
 { 
+   resizeIntegerScaling();
 	mYabauseGL->swapBuffers(); 
 	mYabauseGL->makeCurrent();
 }
 
 void UIYabause::appendLog( const char* s )
 {
+	if (! mCanLog)
+	{
+		qWarning( "%s", s );
+		return;
+	}
+
 	teLog->moveCursor( QTextCursor::End );
 	teLog->append( s );
 
@@ -322,8 +429,7 @@ void UIYabause::sizeRequested( const QSize& s )
 	int width, height;
 	if (s.isNull())
 	{
-		width=640;
-		height=480;
+		return;
 	}
 	else
 	{
@@ -340,9 +446,41 @@ void UIYabause::sizeRequested( const QSize& s )
 		height += menubar->height();
 	if (vs->value( "View/Toolbar" ).toInt() != BD_ALWAYSHIDE)
 		height += toolBar->height();
+
 	resize( width, height ); 
 }
 
+void UIYabause::fixAspectRatio( int width )
+{
+	int aspectRatio = QtYabause::volatileSettings()->value( "Video/AspectRatio").toInt();
+
+	switch( aspectRatio )
+	{
+		case 0:
+			setMaximumSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX );
+			setMinimumSize( 0,0 );
+			break;
+		case 1:
+		case 2:
+		{
+			int heightOffset = toolBar->height()+menubar->height();
+			int height;
+
+			if ( aspectRatio == 1 )
+				height = 3 * ((float) width / 4);
+			else
+				height = 9 * ((float) width / 16);
+
+			mouseYRatio = 240.0 / (float)height * 2.0 * (float)mouseSensitivity / 100.0;
+
+         adjustHeight(height);
+
+			setFixedHeight( height );
+			break;
+		}
+	}
+}
+
 void UIYabause::getSupportedResolutions()
 {
 #if defined Q_OS_WIN
@@ -477,8 +615,12 @@ void UIYabause::fullscreenRequested( bool f )
 #endif
 		VolatileSettings* vs = QtYabause::volatileSettings();
 
+		setMaximumSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX );
+		setMinimumSize( 0,0 );
+
 		toggleFullscreen(vs->value("Video/FullscreenWidth").toInt(), vs->value("Video/FullscreenHeight").toInt(), 
 						f, vs->value("Video/VideoFormat").toInt());
+
 		showFullScreen();
 
 		if ( vs->value( "View/Menubar" ).toInt() == BD_HIDEFS )
@@ -598,11 +740,19 @@ void UIYabause::on_aFileSettings_triggered()
 			newhash[key] = ss->value(key);
 		}
 		if(newhash["General/Bios"]!=hash["General/Bios"] ||
+			newhash["General/EnableEmulatedBios"]!=hash["General/EnableEmulatedBios"] ||
 			newhash["Advanced/Region"]!=hash["Advanced/Region"] ||
 			newhash["Cartridge/Type"]!=hash["Cartridge/Type"] ||
 			newhash["Memory/Path"]!=hash["Memory/Path"] ||
-			newhash["MpegROM/Path" ]!=hash["MpegROM/Path" ] ||
-			newhash["Advanced/SH2Interpreter" ]!=hash["Advanced/SH2Interpreter" ] ||
+			newhash["SH1ROM/Path"]!=hash["SH1ROM/Path"] ||
+			newhash["MpegROM/Path"]!=hash["MpegROM/Path" ] ||
+			newhash["Advanced/SH1Interpreter"]!=hash["Advanced/SH1Interpreter"] ||
+			newhash["Advanced/EnableCDBlockLLE"]!=hash["Advanced/EnableCDBlockLLE"] ||
+         newhash["Advanced/EnableSh2DmaTiming"] != hash["Advanced/EnableSh2DmaTiming"] ||
+         newhash["Advanced/EnableScuDmaTiming"] != hash["Advanced/EnableScuDmaTiming"] ||
+         newhash["Advanced/EnableSh2Cache"] != hash["Advanced/EnableSh2Cache"] ||
+			newhash["Advanced/SH2Interpreter"]!=hash["Advanced/SH2Interpreter"] ||
+         newhash["Advanced/68kCore"] != hash["Advanced/68kCore"] ||
 			newhash["General/CdRom"]!=hash["General/CdRom"] ||
 			newhash["General/CdRomISO"]!=hash["General/CdRomISO"] ||
 			newhash["General/ClockSync"]!=hash["General/ClockSync"] ||
@@ -628,14 +778,37 @@ void UIYabause::on_aFileSettings_triggered()
 		
 		if(newhash["General/ShowFPS"] != hash["General/ShowFPS"])
 			SetOSDToggle(newhash["General/ShowFPS"].toBool());
+
+		if (newhash["General/EnableMultiThreading"] != hash["General/EnableMultiThreading"] ||
+			 newhash["General/NumThreads"] != hash["General/NumThreads"])
+		{
+			if (newhash["General/EnableMultiThreading"].toBool())
+			{
+				int num = newhash["General/NumThreads"].toInt() < 1 ? 1 : newhash["General/NumThreads"].toInt();
+				VIDSoftSetVdp1ThreadEnable(num == 1 ? 0 : 1);
+				VIDSoftSetNumLayerThreads(num);
+				VIDSoftSetNumPriorityThreads(num);
+			}
+			else
+			{
+				VIDSoftSetVdp1ThreadEnable(0);
+				VIDSoftSetNumLayerThreads(1);
+				VIDSoftSetNumPriorityThreads(1);
+			}
+		}
+
 		
 		if (newhash["Sound/SoundCore"] != hash["Sound/SoundCore"])
 			ScspChangeSoundCore(newhash["Sound/SoundCore"].toInt());
-		
+
+      if (newhash["Sound/NewScsp"].toBool() != hash["Sound/NewScsp"])
+         scsp_set_use_new(newhash["Sound/NewScsp"].toInt());
+
 		if (newhash["Video/WindowWidth"] != hash["Video/WindowWidth"] || newhash["Video/WindowHeight"] != hash["Video/WindowHeight"] ||
           newhash["View/Menubar"] != hash["View/Menubar"] || newhash["View/Toolbar"] != hash["View/Toolbar"] || 
 			 newhash["Input/GunMouseSensitivity"] != hash["Input/GunMouseSensitivity"])
 			sizeRequested(QSize(newhash["Video/WindowWidth"].toInt(),newhash["Video/WindowHeight"].toInt()));
+		fixAspectRatio( rect().width() );
 		
 		if (newhash["Video/FullscreenWidth"] != hash["Video/FullscreenWidth"] || 
 			newhash["Video/FullscreenHeight"] != hash["Video/FullscreenHeight"] ||
@@ -658,7 +831,7 @@ void UIYabause::on_aFileSettings_triggered()
 void UIYabause::on_aFileOpenISO_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	const QString fn = CommonDialogs::getOpenFileName( QtYabause::volatileSettings()->value( "Recents/ISOs" ).toString(), QtYabause::translate( "Select your iso/cue/bin file" ), QtYabause::translate( "CD Images (*.iso *.cue *.bin *.mds)" ) );
+	const QString fn = CommonDialogs::getOpenFileName( QtYabause::volatileSettings()->value( "Recents/ISOs" ).toString(), QtYabause::translate( "Select your iso/cue/bin file" ), QtYabause::translate( "CD Images (*.iso *.cue *.bin *.mds *.ccd)" ) );
 	if ( !fn.isEmpty() )
 	{
 		VolatileSettings* vs = QtYabause::volatileSettings();
@@ -670,6 +843,7 @@ void UIYabause::on_aFileOpenISO_triggered()
 		vs->setValue( "autostart", false );
 		vs->setValue( "General/CdRom", ISOCD.id );
 		vs->setValue( "General/CdRomISO", fn );
+      vs->setValue("General/PlaySSF", false);
 		
 		mYabauseThread->pauseEmulation( false, true );
 		
@@ -677,6 +851,31 @@ void UIYabause::on_aFileOpenISO_triggered()
 	}
 }
 
+void UIYabause::on_aFileOpenSSF_triggered()
+{
+   YabauseLocker locker(mYabauseThread);
+
+   const QString fn = CommonDialogs::getOpenFileName(
+      QtYabause::volatileSettings()->value("Recents/SSFs").toString(), 
+      QtYabause::translate("Select your ssf file"), 
+      QtYabause::translate("Sega Saturn Sound Format files (*.ssf *.minissf)"));
+
+   if (!fn.isEmpty())
+   {
+      VolatileSettings* vs = QtYabause::volatileSettings();
+
+      QtYabause::settings()->setValue("Recents/SSFs", fn);
+
+      vs->setValue("autostart", false);
+      vs->setValue("General/SSFPath", fn);
+      vs->setValue("General/PlaySSF", true);
+
+      mYabauseThread->pauseEmulation(false, true);
+
+      refreshStatesActions();
+   }
+}
+
 void UIYabause::on_aFileOpenCDRom_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
@@ -696,6 +895,7 @@ void UIYabause::on_aFileOpenCDRom_triggered()
 		vs->setValue( "autostart", false );
 		vs->setValue( "General/CdRom", QtYabause::defaultCDCore().id );
 		vs->setValue( "General/CdRomISO", fn );
+      vs->setValue("General/PlaySSF", false);
 
 		mYabauseThread->pauseEmulation( false, true );
 
@@ -764,12 +964,22 @@ void UIYabause::on_aFileScreenshot_triggered()
 	QImage screenshot = mYabauseGL->grabFrameBuffer();
 	
 	// request a file to save to to user
-	const QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for your screenshot" ), filters.join( ";;" ) );
+	QString s = CommonDialogs::getSaveFileName( QString(), QtYabause::translate( "Choose a location for your screenshot" ), filters.join( ";;" ) );
+
+	// if the user didn't provide a filename extension, we force it to png
+	QFileInfo qfi( s );
+	if ( qfi.suffix().isEmpty() )
+		s += ".png";
 	
 	// write image if ok
 	if ( !s.isEmpty() )
-		if ( !screenshot.save( s ) )
-			CommonDialogs::information( QtYabause::translate( "An error occur while writing the screenshot." ) );
+	{
+		QImageWriter iw( s );
+		if ( !iw.write( screenshot ))
+		{
+			CommonDialogs::information( QtYabause::translate( "An error occur while writing the screenshot: " + iw.errorString()) );
+		}
+	}
 }
 
 void UIYabause::on_aFileQuit_triggered()
@@ -869,7 +1079,14 @@ void UIYabause::on_aViewLayerRBG0_triggered()
 void UIYabause::on_aViewFullscreen_triggered( bool b )
 {
 	fullscreenRequested( b );
-	//ToggleFullScreen();
+}
+
+void UIYabause::breakpointHandlerSH1(bool displayMessage)
+{
+	YabauseLocker locker( mYabauseThread );
+	if (displayMessage)
+		CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
+	UIDebugSH2(UIDebugCPU::PROC_SH1, mYabauseThread, this ).exec();
 }
 
 void UIYabause::breakpointHandlerMSH2(bool displayMessage)
@@ -877,7 +1094,7 @@ void UIYabause::breakpointHandlerMSH2(bool displayMessage)
 	YabauseLocker locker( mYabauseThread );
 	if (displayMessage)
 		CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
-	UIDebugSH2( true, mYabauseThread, this ).exec();
+	UIDebugSH2(UIDebugCPU::PROC_MSH2, mYabauseThread, this ).exec();
 }
 
 void UIYabause::breakpointHandlerSSH2(bool displayMessage)
@@ -885,7 +1102,7 @@ void UIYabause::breakpointHandlerSSH2(bool displayMessage)
 	YabauseLocker locker( mYabauseThread );
 	if (displayMessage)
 		CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
-	UIDebugSH2( false, mYabauseThread, this ).exec();
+	UIDebugSH2(UIDebugCPU::PROC_SSH2, mYabauseThread, this ).exec();
 }
 
 void UIYabause::breakpointHandlerM68K()
@@ -902,16 +1119,23 @@ void UIYabause::breakpointHandlerSCUDSP()
 	UIDebugSCUDSP( mYabauseThread, this ).exec();
 }
 
+void UIYabause::breakpointHandlerSCSPDSP()
+{
+	YabauseLocker locker( mYabauseThread );
+	CommonDialogs::information( QtYabause::translate( "Breakpoint Reached" ) );
+	UIDebugSCSPDSP( mYabauseThread, this ).exec();
+}
+
 void UIYabause::on_aViewDebugMSH2_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	UIDebugSH2( true, mYabauseThread, this ).exec();
+	UIDebugSH2( UIDebugCPU::PROC_MSH2, mYabauseThread, this ).exec();
 }
 
 void UIYabause::on_aViewDebugSSH2_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	UIDebugSH2( false, mYabauseThread, this ).exec();
+	UIDebugSH2( UIDebugCPU::PROC_SSH2, mYabauseThread, this ).exec();
 }
 
 void UIYabause::on_aViewDebugVDP1_triggered()
@@ -944,14 +1168,45 @@ void UIYabause::on_aViewDebugSCSP_triggered()
 	UIDebugSCSP( this ).exec();
 }
 
+void UIYabause::on_aViewDebugSCSPChan_triggered()
+{
+   if (use_new_scsp)
+      UIDebugSCSPChan(this).exec();
+   else
+      CommonDialogs::information( QtYabause::translate( "Only available with new scsp code(USE_NEW_SCSP=1)" ) );
+}
+
+void UIYabause::on_aViewDebugSCSPDSP_triggered()
+{
+	YabauseLocker locker( mYabauseThread );
+	UIDebugSCSPDSP( mYabauseThread, this ).exec();
+}
+
+void UIYabause::on_aViewDebugSH1_triggered()
+{
+	YabauseLocker locker( mYabauseThread );
+	UIDebugSH2( UIDebugCPU::PROC_SH1, mYabauseThread, this ).exec();
+}
+
 void UIYabause::on_aViewDebugMemoryEditor_triggered()
 {
 	YabauseLocker locker( mYabauseThread );
-	UIMemoryEditor( mYabauseThread, this ).exec();
+	UIMemoryEditor( UIDebugCPU::PROC_MSH2, mYabauseThread, this ).exec();
 }
 
-void UIYabause::on_aHelpEmuCompatibility_triggered()
-{ QDesktopServices::openUrl( QUrl( aHelpEmuCompatibility->statusTip() ) ); }
+void UIYabause::on_aTraceLogging_triggered( bool toggled )
+{
+#ifdef SH2_TRACE
+	SH2SetInsTracing(toggled? 1 : 0);
+#endif
+	return;
+}
+
+void UIYabause::on_aHelpDocumentation_triggered()
+{ QDesktopServices::openUrl( QUrl( aHelpDocumentation->statusTip() ) ); }
+
+void UIYabause::on_aHelpCompatibilityList_triggered()
+{ QDesktopServices::openUrl( QUrl( aHelpCompatibilityList->statusTip() ) ); }
 
 void UIYabause::on_aHelpAbout_triggered()
 {
@@ -1034,4 +1289,3 @@ void UIYabause::toggleEmulateMouse( bool enable )
 {
 	emulateMouse = enable;
 }
-
