@@ -74,6 +74,7 @@ class UIYabause : public QMainWindow, public Ui::UIYabause
 	
 public:
 	UIYabause( QWidget* parent = 0 );
+	~UIYabause();
 
 	void swapBuffers();
 	virtual bool eventFilter( QObject* o, QEvent* e );
@@ -83,11 +84,14 @@ protected:
 	YabauseThread* mYabauseThread;
 	QDockWidget* mLogDock;
 	QTextEdit* teLog;
+	bool mCanLog;
 	bool mInit;
 	QList <cheatsearch_struct> search;
 	int searchType;
 	QList <supportedRes_struct> supportedResolutions;
 	int oldMouseX, oldMouseY;
+	bool mouseCaptured;
+
 	float mouseXRatio, mouseYRatio;
 	int mouseSensitivity;
 	bool emulateMouse;
@@ -99,9 +103,11 @@ protected:
 	virtual void closeEvent( QCloseEvent* event );
 	virtual void keyPressEvent( QKeyEvent* event );
 	virtual void keyReleaseEvent( QKeyEvent* event );
+	virtual void leaveEvent(QEvent * event);
 	virtual void mousePressEvent( QMouseEvent* event );
 	virtual void mouseReleaseEvent( QMouseEvent* event );
 	virtual void mouseMoveEvent( QMouseEvent* event );
+	virtual void resizeEvent( QResizeEvent* event );
 
 public slots:
 	void appendLog( const char* msg );
@@ -111,23 +117,29 @@ public slots:
 	void cursorRestore();
 	void toggleEmulateMouse( bool enable );
 
+	void breakpointHandlerSH1(bool displayMessage);
 	void breakpointHandlerMSH2(bool displayMessage);
 	void breakpointHandlerSSH2(bool displayMessage);
 	void breakpointHandlerM68K();
 	void breakpointHandlerSCUDSP();
+	void breakpointHandlerSCSPDSP();
 protected slots:
 	void errorReceived( const QString& error, bool internal = true );
 	void sizeRequested( const QSize& size );
+	void fixAspectRatio( int width );
 	void getSupportedResolutions();
 	int isResolutionValid( int width, int height, int bpp, int freq );
 	int findBestVideoFreq( int width, int height, int bpp, int videoFormat );
 	void toggleFullscreen( int width, int height, bool f, int videoFormat );
 	void fullscreenRequested( bool fullscreen );
 	void refreshStatesActions();
+   void adjustHeight(int & height);
+   void resizeIntegerScaling();
 	// file menu
 	void on_aFileSettings_triggered();
 	void on_aFileOpenISO_triggered();
 	void on_aFileOpenCDRom_triggered();
+   void on_aFileOpenSSF_triggered();
 	void on_mFileSaveState_triggered( QAction* );
 	void on_mFileLoadState_triggered( QAction* );
 	void on_aFileSaveStateAs_triggered();
@@ -159,11 +171,16 @@ protected slots:
 	void on_aViewDebugVDP1_triggered();
 	void on_aViewDebugVDP2_triggered();
 	void on_aViewDebugM68K_triggered();
-	void on_aViewDebugSCSP_triggered();
+   void on_aViewDebugSCSP_triggered();
+   void on_aViewDebugSCSPChan_triggered();
+	void on_aViewDebugSCSPDSP_triggered();
 	void on_aViewDebugSCUDSP_triggered();
+	void on_aViewDebugSH1_triggered();
 	void on_aViewDebugMemoryEditor_triggered();
+	void on_aTraceLogging_triggered( bool toggled );
 	// help menu
-	void on_aHelpEmuCompatibility_triggered();
+   void on_aHelpDocumentation_triggered();
+	void on_aHelpCompatibilityList_triggered();
 	void on_aHelpAbout_triggered();
 	// toolbar
 	void on_aSound_triggered();
