@@ -27,6 +27,7 @@
 #define SCSP_H
 
 #include "core.h"
+#include "memory.h"
 
 #define SNDCORE_DEFAULT -1
 #define SNDCORE_DUMMY   0
@@ -48,6 +49,11 @@ typedef struct
    void (*MuteAudio)(void);
    void (*UnMuteAudio)(void);
    void (*SetVolume)(int volume);
+#ifdef USE_SCSPMIDI
+	int (*MidiChangePorts)(int inport, int outport);
+	u8 (*MidiIn)(int *isdata);
+	int (*MidiOut)(u8 data);
+#endif
 } SoundInterface_struct;
 
 typedef struct
@@ -79,6 +85,7 @@ typedef struct
 extern SoundInterface_struct SNDDummy;
 extern SoundInterface_struct SNDWave;
 extern u8 *SoundRam;
+extern int use_new_scsp;
 
 u8 FASTCALL SoundRamReadByte(u32 addr);
 u16 FASTCALL SoundRamReadWord(u32 addr);
@@ -87,9 +94,15 @@ void FASTCALL SoundRamWriteByte(u32 addr, u8 val);
 void FASTCALL SoundRamWriteWord(u32 addr, u16 val);
 void FASTCALL SoundRamWriteLong(u32 addr, u32 val);
 
+u8 FASTCALL Sh2SoundRamReadByte(SH2_struct *sh, u32 addr);
+u16 FASTCALL Sh2SoundRamReadWord(SH2_struct *sh, u32 addr);
+u32 FASTCALL Sh2SoundRamReadLong(SH2_struct *sh, u32 addr);
+void FASTCALL Sh2SoundRamWriteByte(SH2_struct *sh, u32 addr, u8 val);
+void FASTCALL Sh2SoundRamWriteWord(SH2_struct *sh, u32 addr, u16 val);
+void FASTCALL Sh2SoundRamWriteLong(SH2_struct *sh, u32 addr, u32 val);
+
 int ScspInit(int coreid);
 int ScspChangeSoundCore(int coreid);
-void ScspSetFrameAccurate(int on);
 void ScspDeInit(void);
 void M68KStart(void);
 void M68KStop(void);
@@ -99,22 +112,33 @@ void M68KExec(s32 cycles);
 void ScspExec(void);
 void ScspConvert32uto16s(s32 *srcL, s32 *srcR, s16 *dst, u32 len);
 void ScspReceiveCDDA(const u8 *sector);
+void ScspReceiveMpeg (const u8 *samples, int len);
 int SoundSaveState(FILE *fp);
 int SoundLoadState(FILE *fp, int version, int size);
 void ScspSlotDebugStats(u8 slotnum, char *outstring);
 void ScspCommonControlRegisterDebugStats(char *outstring);
 int ScspSlotDebugSaveRegisters(u8 slotnum, const char *filename);
+u32 ScspSlotDebugAudio (u32 *workbuf, s16 *buf, u32 len);
+void ScspSlotResetDebug(u8 slotnum);
 int ScspSlotDebugAudioSaveWav(u8 slotnum, const char *filename);
 void ScspMuteAudio(int flags);
 void ScspUnMuteAudio(int flags);
 void ScspSetVolume(int volume);
 
-void FASTCALL scsp_w_b(u32, u8);
-void FASTCALL scsp_w_w(u32, u16);
-void FASTCALL scsp_w_d(u32, u32);
-u8 FASTCALL scsp_r_b(u32);
-u16 FASTCALL scsp_r_w(u32);
-u32 FASTCALL scsp_r_d(u32);
+
+u8 FASTCALL ScspReadByte(u32 addr);
+void FASTCALL ScspWriteByte(u32 addr, u8 val);
+u16 FASTCALL ScspReadWord(u32 addr);
+void FASTCALL ScspWriteWord(u32 addr, u16 val);
+u32 FASTCALL ScspReadLong(u32 addr);
+void FASTCALL ScspWriteLong(u32 addr, u32 val);
+
+u8 FASTCALL Sh2ScspReadByte(SH2_struct *sh, u32 addr);
+void FASTCALL Sh2ScspWriteByte(SH2_struct *sh, u32 addr, u8 val);
+u16 FASTCALL Sh2ScspReadWord(SH2_struct *sh, u32 addr);
+void FASTCALL Sh2ScspWriteWord(SH2_struct *sh, u32 addr, u16 val);
+u32 FASTCALL Sh2ScspReadLong(SH2_struct *sh, u32 addr);
+void FASTCALL Sh2ScspWriteLong(SH2_struct *sh, u32 addr, u32 val);
 
 void scsp_init(u8 *scsp_ram, void (*sint_hand)(u32), void (*mint_hand)(void));
 void scsp_shutdown(void);
@@ -128,6 +152,7 @@ void scsp_update(s32 *bufL, s32 *bufR, u32 len);
 void scsp_update_monitor(void);
 void scsp_update_timer(u32 len);
 
+u32 FASTCALL c68k_byte_read(const u32 adr);
 u32 FASTCALL c68k_word_read(const u32 adr);
 
 void M68KStep(void);
@@ -142,4 +167,13 @@ int M68KDelCodeBreakpoint(u32 addr);
 m68kcodebreakpoint_struct *M68KGetBreakpointList(void);
 void M68KClearCodeBreakpoints(void);
 
+void scsp_debug_instrument_get_data(int i, u32 * sa, int * is_muted);
+void scsp_debug_instrument_set_mute(u32 sa, int mute);
+void scsp_debug_instrument_clear();
+void scsp_debug_get_envelope(int chan, int * env, int * state);
+void scsp_debug_set_mode(int mode);
+void scsp_set_use_new(int which);
+void new_scsp_exec(s32 cycles);
+
+extern int use_new_scsp;
 #endif
