@@ -31,7 +31,7 @@
 /*************************************************************************/
 
 /* Define BINARY_LOG to log traces in a binary format (faster than text) */
-// #define BINARY_LOG
+//#define BINARY_LOG
 
 /* Define GZIP_LOG to compress log as it's created */
 #ifdef __linux__
@@ -49,12 +49,19 @@ static const u64 trace_stop  = 2800000000ULL;  // Last cycle to trace + 1
 
 /*----------------------------------*/
 
+static int is_ins_enabled = 0;
+
 static FILE *logfile;                // Trace log file
 static u64 cycle_accum = 0;     // Global cycle accumulator
 static u64 current_cycles = 0;  // Cycle count on last call to sh2_trace()
 
 /*************************************************************************/
 
+void SH2SetInsTracing(int enable)
+{
+   is_ins_enabled = enable;
+}
+
 FASTCALL u64 sh2_cycle_count(void)
 {
     return current_cycles;
@@ -168,6 +175,9 @@ static INLINE void HEXIT(char * const ptr, u32 val, int ndigits)
 
 FASTCALL void sh2_trace(SH2_struct *state, u32 address)
 {
+    if (!is_ins_enabled)
+        return;
+
     current_cycles = cycle_accum + state->cycles;
 
     if (current_cycles < trace_start) {
@@ -218,7 +228,7 @@ FASTCALL void sh2_trace(SH2_struct *state, u32 address)
             setvbuf(logfile, NULL, _IOFBF, 65536);
         }
 
-        opcode = MappedMemoryReadWord(address);
+        opcode = MappedMemoryReadWordNocache(state, address);
 
 #ifdef BINARY_LOG
 
@@ -237,7 +247,7 @@ FASTCALL void sh2_trace(SH2_struct *state, u32 address)
 
         SH2GetRegisters(state, &state->regs);
 
-        SH2Disasm(address, opcode, 0, buf);
+        SH2Disasm(address, opcode, 0, &state->regs, buf);
         fprintf(logfile, "[%c] %08X: %04X  %-44s [%12llu]\n",
                 state==SSH2 ? 'S' : 'M', (int)address, (int)opcode, buf+12,
                 (unsigned long long)current_cycles);
