@@ -33,6 +33,19 @@ enum {
    YAB_THREAD_NETLINKLISTENER,
    YAB_THREAD_NETLINKCONNECT,
    YAB_THREAD_NETLINKCLIENT,
+   YAB_THREAD_OPENAL,
+   YAB_THREAD_VIDSOFT_LAYER_NBG3,
+   YAB_THREAD_VIDSOFT_LAYER_NBG2,
+   YAB_THREAD_VIDSOFT_LAYER_NBG1,
+   YAB_THREAD_VIDSOFT_LAYER_NBG0,
+   YAB_THREAD_VIDSOFT_LAYER_RBG0,
+   YAB_THREAD_VIDSOFT_VDP1,
+   YAB_THREAD_VIDSOFT_PRIORITY_0,
+   YAB_THREAD_VIDSOFT_PRIORITY_1,
+   YAB_THREAD_VIDSOFT_PRIORITY_2,
+   YAB_THREAD_VIDSOFT_PRIORITY_3,
+   YAB_THREAD_VIDSOFT_PRIORITY_4,
+   YAB_THREAD_VIDSOFT_LAYER_SPRITE,
    YAB_NUM_THREADS      // Total number of subthreads
 };
 
