@@ -42,6 +42,20 @@ void FASTCALL   Vdp2ColorRamWriteByte(u32, u8);
 void FASTCALL   Vdp2ColorRamWriteWord(u32, u16);
 void FASTCALL   Vdp2ColorRamWriteLong(u32, u32);
 
+u8 FASTCALL     Sh2Vdp2RamReadByte(SH2_struct *, u32);
+u16 FASTCALL    Sh2Vdp2RamReadWord(SH2_struct *, u32);
+u32 FASTCALL    Sh2Vdp2RamReadLong(SH2_struct *, u32);
+void FASTCALL   Sh2Vdp2RamWriteByte(SH2_struct *, u32, u8);
+void FASTCALL   Sh2Vdp2RamWriteWord(SH2_struct *, u32, u16);
+void FASTCALL   Sh2Vdp2RamWriteLong(SH2_struct *, u32, u32);
+
+u8 FASTCALL     Sh2Vdp2ColorRamReadByte(SH2_struct *, u32);
+u16 FASTCALL    Sh2Vdp2ColorRamReadWord(SH2_struct *, u32);
+u32 FASTCALL    Sh2Vdp2ColorRamReadLong(SH2_struct *, u32);
+void FASTCALL   Sh2Vdp2ColorRamWriteByte(SH2_struct *, u32, u8);
+void FASTCALL   Sh2Vdp2ColorRamWriteWord(SH2_struct *, u32, u16);
+void FASTCALL   Sh2Vdp2ColorRamWriteLong(SH2_struct *, u32, u32);
+
 typedef struct {
    u16 TVMD;   // 0x25F80000
    u16 EXTEN;  // 0x25F80002
@@ -361,6 +375,15 @@ typedef struct {
 
 extern Vdp2Internal_struct Vdp2Internal;
 extern u64 lastticks;
+extern int vdp2_is_odd_frame;
+extern Vdp2 Vdp2Lines[270];
+
+struct CellScrollData
+{
+   u32 data[88];//(352/8) * 2 screens
+};
+
+extern struct CellScrollData cell_scroll_data[270];
 
 // struct for Vdp2 part that shouldn't be saved
 typedef struct {
@@ -387,6 +410,13 @@ void FASTCALL   Vdp2WriteByte(u32, u8);
 void FASTCALL   Vdp2WriteWord(u32, u16);
 void FASTCALL   Vdp2WriteLong(u32, u32);
 
+u8 FASTCALL     Sh2Vdp2ReadByte(SH2_struct *, u32);
+u16 FASTCALL    Sh2Vdp2ReadWord(SH2_struct *, u32);
+u32 FASTCALL    Sh2Vdp2ReadLong(SH2_struct *, u32);
+void FASTCALL   Sh2Vdp2WriteByte(SH2_struct *, u32, u8);
+void FASTCALL   Sh2Vdp2WriteWord(SH2_struct *, u32, u16);
+void FASTCALL   Sh2Vdp2WriteLong(SH2_struct *, u32, u32);
+
 int Vdp2SaveState(FILE *fp);
 int Vdp2LoadState(FILE *fp, int version, int size);
 
@@ -399,6 +429,6 @@ void ToggleFullScreen(void);
 void EnableAutoFrameSkip(void);
 void DisableAutoFrameSkip(void);
 
-Vdp2 * Vdp2RestoreRegs(int line);
+Vdp2 * Vdp2RestoreRegs(int line, Vdp2* lines);
 
 #endif
