@@ -41,7 +41,7 @@ static INLINE void Vdp2GetPlaneSize(int planedata, int *planew, int *planeh)
          *planew = 2;
          *planeh = 1;
          break;
-      case 2:
+      case 3:
          *planew = *planeh = 2;
          break;
       default:
@@ -141,7 +141,7 @@ static INLINE char *AddBitmapInfoString(char *outstring, int wh, int palnum, int
 static void CalcWindowCoordinates(int num, int *hstart, int *vstart, int *hend, int *vend)
 {
    clipping_struct clip;
-   ReadWindowCoordinates(num, &clip);
+   ReadWindowCoordinates(num, &clip, Vdp2Regs);
    *hstart = clip.xstart;
    *vstart = clip.ystart;
    *hend = clip.xend;
@@ -246,23 +246,7 @@ static INLINE char *AddMapInfo(char *outstring, int patternwh, u16 PNC, u8 PLSZ,
    else
       patterndatasize = 2;
 
-   switch(PLSZ)
-   {
-      case 0:
-         planew = planeh = 1;
-         break;
-      case 1:
-         planew = 2;
-         planeh = 1;
-         break;
-      case 2:
-         planew = planeh = 2;
-         break;
-      default: // Not sure what 0x3 does
-         planew = planeh = 1;
-         break;
-   }
-
+   Vdp2GetPlaneSize(PLSZ, &planew, &planeh);
    deca = planeh + planew - 2;
    multi = planeh * planew;
 
@@ -836,6 +820,7 @@ void Vdp2DebugStatsNBG0(char *outstring, int *isenabled)
          // Screen scroll values
          AddString(outstring, "Screen Scroll x = %f, y = %f\r\n", (float)(reg->getLong(0x70) & 0x7FFFF00) / 65536, (float)(reg->getLong(0x74) & 0x7FFFF00) / 65536);
 */
+         AddString(outstring, "Screen Scroll x = %d, y = %d\r\n", - ((Vdp2Regs->SCXIN0 & 0x7FF) % 512), - ((Vdp2Regs->SCYIN0 & 0x7FF) % 512));
       
          // Coordinate Increments
          AddString(outstring, "Coordinate Increments x = %f, y = %f\r\n", (float) 65536 / (Vdp2Regs->ZMXN0.all & 0x7FF00), (float) 65536 / (Vdp2Regs->ZMYN0.all & 0x7FF00));
@@ -1052,6 +1037,7 @@ void Vdp2DebugStatsNBG1(char *outstring, int *isenabled)
       // Screen scroll values
       AddString(outstring, "Screen Scroll x = %f, y = %f\r\n", (float)(reg->getLong(0x80) & 0x7FFFF00) / 65536, (float)(reg->getLong(0x84) & 0x7FFFF00) / 65536);
 */
+         AddString(outstring, "Screen Scroll x = %d, y = %d\r\n", - ((Vdp2Regs->SCXIN1 & 0x7FF) % 512), - ((Vdp2Regs->SCYIN1 & 0x7FF) % 512));
 
       // Coordinate Increments
       AddString(outstring, "Coordinate Increments x = %f, y = %f\r\n", (float) 65536 / (Vdp2Regs->ZMXN1.all & 0x7FF00), (float) 65536 / (Vdp2Regs->ZMXN1.all & 0x7FF00));
@@ -1661,6 +1647,7 @@ pixel_t *Vdp2DebugTexture(u32 screen, int * w, int * h)
    pixel_t * bitmap;
 
    TitanInit();
+   TitanSetBlendingMode(TITAN_BLEND_TOP);
    VIDSoftVdp2DrawScreen(screen);
 
    if ((bitmap = (pixel_t *)calloc(sizeof(pixel_t), 704 * 512)) == NULL)
