@@ -29,24 +29,24 @@
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2NBG0PlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2NBG0PlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFN & 0x7) << 6;
+   u32 offset = (regs->MPOFN & 0x7) << 6;
    u32 tmp=0;
 
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABN0 & 0xFF);
+         tmp = offset | (regs->MPABN0 & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABN0 >> 8);
+         tmp = offset | (regs->MPABN0 >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDN0 & 0xFF);
+         tmp = offset | (regs->MPCDN0 & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDN0 >> 8);
+         tmp = offset | (regs->MPCDN0 >> 8);
          break;
    }
 
@@ -55,24 +55,24 @@ void FASTCALL Vdp2NBG0PlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2NBG1PlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2NBG1PlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFN & 0x70) << 2;
+   u32 offset = (regs->MPOFN & 0x70) << 2;
    u32 tmp=0;
 
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABN1 & 0xFF);
+         tmp = offset | (regs->MPABN1 & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABN1 >> 8);
+         tmp = offset | (regs->MPABN1 >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDN1 & 0xFF);
+         tmp = offset | (regs->MPCDN1 & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDN1 >> 8);
+         tmp = offset | (regs->MPCDN1 >> 8);
          break;
    }
 
@@ -81,24 +81,24 @@ void FASTCALL Vdp2NBG1PlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2NBG2PlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2NBG2PlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFN & 0x700) >> 2;
+   u32 offset = (regs->MPOFN & 0x700) >> 2;
    u32 tmp=0;
 
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABN2 & 0xFF);
+         tmp = offset | (regs->MPABN2 & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABN2 >> 8);
+         tmp = offset | (regs->MPABN2 >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDN2 & 0xFF);
+         tmp = offset | (regs->MPCDN2 & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDN2 >> 8);
+         tmp = offset | (regs->MPCDN2 >> 8);
          break;
    }
 
@@ -107,24 +107,24 @@ void FASTCALL Vdp2NBG2PlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2NBG3PlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2NBG3PlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFN & 0x7000) >> 6;
+   u32 offset = (regs->MPOFN & 0x7000) >> 6;
    u32 tmp=0;
 
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABN3 & 0xFF);
+         tmp = offset | (regs->MPABN3 & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABN3 >> 8);
+         tmp = offset | (regs->MPABN3 >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDN3 & 0xFF);
+         tmp = offset | (regs->MPCDN3 & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDN3 >> 8);
+         tmp = offset | (regs->MPCDN3 >> 8);
          break;
    }
 
@@ -133,146 +133,146 @@ void FASTCALL Vdp2NBG3PlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void Vdp2ReadRotationTable(int which, vdp2rotationparameter_struct *parameter)
+void Vdp2ReadRotationTable(int which, vdp2rotationparameter_struct *parameter, Vdp2* regs, u8* ram)
 {
    s32 i;
    u32 addr;
 
-   addr = Vdp2Regs->RPTA.all << 1;
+   addr = regs->RPTA.all << 1;
 
    if (which == 0)
    {
       // Rotation Parameter A
       addr &= 0x000FFF7C;
-      parameter->coefenab = Vdp2Regs->KTCTL & 0x1;
-      parameter->screenover = (Vdp2Regs->PLSZ >> 10) & 0x3;
+      parameter->coefenab = regs->KTCTL & 0x1;
+      parameter->screenover = (regs->PLSZ >> 10) & 0x3;
    }
    else
    {
       // Rotation Parameter B
       addr = (addr & 0x000FFFFC) | 0x00000080;
-      parameter->coefenab = Vdp2Regs->KTCTL & 0x100;
-      parameter->screenover = (Vdp2Regs->PLSZ >> 14) & 0x3;
+      parameter->coefenab = regs->KTCTL & 0x100;
+      parameter->screenover = (regs->PLSZ >> 14) & 0x3;
    }
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Xst = (float) (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Yst = (float) (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Zst = (float) (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaXst = (float) (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaYst = (float) (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaX = (float) (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaY = (float) (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->A = (float) (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->B = (float) (signed) ((i & 0x000FFFC0) | ((i & 0x00080000) ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->C = (float) (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->D = (float) (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->E = (float) (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->F = (float) (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Px = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Py = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Pz = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cx = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cy = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cz = (float) (signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Mx = (float) (signed) ((i & 0x3FFFFFC0) | (i & 0x20000000 ? 0xE0000000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->My = (float) (signed) ((i & 0x3FFFFFC0) | (i & 0x20000000 ? 0xE0000000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->kx = (float) (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000)) / 65536;
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->ky = (float) (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000)) / 65536;
    addr += 4;
 
    if (parameter->coefenab)
    {
       // Read in coefficient values
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->KAst = (float)(unsigned)(i & 0xFFFFFFC0) / 65536;
       addr += 4;
 
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->deltaKAst = (float) (signed) ((i & 0x03FFFFC0) | (i & 0x02000000 ? 0xFE000000 : 0x00000000)) / 65536;
       addr += 4;     
 
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->deltaKAx = (float) (signed) ((i & 0x03FFFFC0) | (i & 0x02000000 ? 0xFE000000 : 0x00000000)) / 65536;
       addr += 4;
 
       if (which == 0)
       {
-         parameter->coefdatasize = (Vdp2Regs->KTCTL & 0x2 ? 2 : 4);
-         parameter->coeftbladdr = ((Vdp2Regs->KTAOF & 0x7) * 0x10000 + (int)(parameter->KAst)) * parameter->coefdatasize;
-         parameter->coefmode = (Vdp2Regs->KTCTL >> 2) & 0x3;
+         parameter->coefdatasize = (regs->KTCTL & 0x2 ? 2 : 4);
+         parameter->coeftbladdr = ((regs->KTAOF & 0x7) * 0x10000 + (int)(parameter->KAst)) * parameter->coefdatasize;
+         parameter->coefmode = (regs->KTCTL >> 2) & 0x3;
       }
       else
       {
-         parameter->coefdatasize = (Vdp2Regs->KTCTL & 0x200 ? 2 : 4);
-         parameter->coeftbladdr = (((Vdp2Regs->KTAOF >> 8) & 0x7) * 0x10000 + (int)(parameter->KAst)) * parameter->coefdatasize;
-         parameter->coefmode = (Vdp2Regs->KTCTL >> 10) & 0x3;
+         parameter->coefdatasize = (regs->KTCTL & 0x200 ? 2 : 4);
+         parameter->coeftbladdr = (((regs->KTAOF >> 8) & 0x7) * 0x10000 + (int)(parameter->KAst)) * parameter->coefdatasize;
+         parameter->coefmode = (regs->KTCTL >> 10) & 0x3;
       }
    }
    
@@ -292,146 +292,146 @@ void Vdp2ReadRotationTable(int which, vdp2rotationparameter_struct *parameter)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void Vdp2ReadRotationTableFP(int which, vdp2rotationparameterfp_struct *parameter)
+void Vdp2ReadRotationTableFP(int which, vdp2rotationparameterfp_struct *parameter, Vdp2* regs, u8 * ram)
 {
    s32 i;
    u32 addr;
 
-   addr = Vdp2Regs->RPTA.all << 1;
+   addr = regs->RPTA.all << 1;
 
    if (which == 0)
    {
       // Rotation Parameter A
       addr &= 0x000FFF7C;
-      parameter->coefenab = Vdp2Regs->KTCTL & 0x1;
-      parameter->screenover = (Vdp2Regs->PLSZ >> 10) & 0x3;
+      parameter->coefenab = regs->KTCTL & 0x1;
+      parameter->screenover = (regs->PLSZ >> 10) & 0x3;
    }
    else
    {
       // Rotation Parameter B
       addr = (addr & 0x000FFFFC) | 0x00000080;
-      parameter->coefenab = Vdp2Regs->KTCTL & 0x100;
-      parameter->screenover = (Vdp2Regs->PLSZ >> 14) & 0x3;
+      parameter->coefenab = regs->KTCTL & 0x100;
+      parameter->screenover = (regs->PLSZ >> 14) & 0x3;
    }
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Xst = (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Yst = (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Zst = (signed) ((i & 0x1FFFFFC0) | (i & 0x10000000 ? 0xF0000000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaXst = (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaYst = (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaX = (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->deltaY = (signed) ((i & 0x0007FFC0) | (i & 0x00040000 ? 0xFFFC0000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->A = (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->B = (signed) ((i & 0x000FFFC0) | ((i & 0x00080000) ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->C = (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->D = (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->E = (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->F = (signed) ((i & 0x000FFFC0) | (i & 0x00080000 ? 0xFFF80000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Px = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Py = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Pz = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 4;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cx = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cy = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 2;
 
-   i = T1ReadWord(Vdp2Ram, addr);
+   i = T1ReadWord(ram, addr);
    parameter->Cz = tofixed((signed) ((i & 0x3FFF) | (i & 0x2000 ? 0xFFF80000 : 0x00000000)));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->Mx = (signed) ((i & 0x3FFFFFC0) | (i & 0x20000000 ? 0xE0000000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->My = (signed) ((i & 0x3FFFFFC0) | (i & 0x20000000 ? 0xE0000000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->kx = (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000));
    addr += 4;
 
-   i = T1ReadLong(Vdp2Ram, addr);
+   i = T1ReadLong(ram, addr);
    parameter->ky = (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000));
    addr += 4;
 
    if (parameter->coefenab)
    {
       // Read in coefficient values
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->KAst = (unsigned)(i & 0xFFFFFFC0);
       addr += 4;
 
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->deltaKAst = (signed) ((i & 0x03FFFFC0) | (i & 0x02000000 ? 0xFE000000 : 0x00000000));
       addr += 4;     
 
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->deltaKAx = (signed) ((i & 0x03FFFFC0) | (i & 0x02000000 ? 0xFE000000 : 0x00000000));
       addr += 4;
 
       if (which == 0)
       {
-         parameter->coefdatasize = (Vdp2Regs->KTCTL & 0x2 ? 2 : 4);
-         parameter->coeftbladdr = ((Vdp2Regs->KTAOF & 0x7) * 0x10000 + touint(parameter->KAst)) * parameter->coefdatasize;
-         parameter->coefmode = (Vdp2Regs->KTCTL >> 2) & 0x3;
+         parameter->coefdatasize = (regs->KTCTL & 0x2 ? 2 : 4);
+         parameter->coeftbladdr = ((regs->KTAOF & 0x7) * 0x10000 + touint(parameter->KAst)) * parameter->coefdatasize;
+         parameter->coefmode = (regs->KTCTL >> 2) & 0x3;
       }
       else
       {
-         parameter->coefdatasize = (Vdp2Regs->KTCTL & 0x200 ? 2 : 4);
-         parameter->coeftbladdr = (((Vdp2Regs->KTAOF >> 8) & 0x7) * 0x10000 + touint(parameter->KAst)) * parameter->coefdatasize;
-         parameter->coefmode = (Vdp2Regs->KTCTL >> 10) & 0x3;
+         parameter->coefdatasize = (regs->KTCTL & 0x200 ? 2 : 4);
+         parameter->coeftbladdr = (((regs->KTAOF >> 8) & 0x7) * 0x10000 + touint(parameter->KAst)) * parameter->coefdatasize;
+         parameter->coefmode = (regs->KTCTL >> 10) & 0x3;
       }
    }
 
@@ -454,60 +454,60 @@ void Vdp2ReadRotationTableFP(int which, vdp2rotationparameterfp_struct *paramete
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2ParameterAPlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2ParameterAPlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFR & 0x7) << 6;
+   u32 offset = (regs->MPOFR & 0x7) << 6;
    u32 tmp=0;
 
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABRA & 0xFF);
+         tmp = offset | (regs->MPABRA & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABRA >> 8);
+         tmp = offset | (regs->MPABRA >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDRA & 0xFF);
+         tmp = offset | (regs->MPCDRA & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDRA >> 8);
+         tmp = offset | (regs->MPCDRA >> 8);
          break;
       case 4:
-         tmp = offset | (Vdp2Regs->MPEFRA & 0xFF);
+         tmp = offset | (regs->MPEFRA & 0xFF);
          break;
       case 5:
-         tmp = offset | (Vdp2Regs->MPEFRA >> 8);
+         tmp = offset | (regs->MPEFRA >> 8);
          break;
       case 6:
-         tmp = offset | (Vdp2Regs->MPGHRA & 0xFF);
+         tmp = offset | (regs->MPGHRA & 0xFF);
          break;
       case 7:
-         tmp = offset | (Vdp2Regs->MPGHRA >> 8);
+         tmp = offset | (regs->MPGHRA >> 8);
          break;
       case 8:
-         tmp = offset | (Vdp2Regs->MPIJRA & 0xFF);
+         tmp = offset | (regs->MPIJRA & 0xFF);
          break;
       case 9:
-         tmp = offset | (Vdp2Regs->MPIJRA >> 8);
+         tmp = offset | (regs->MPIJRA >> 8);
          break;
       case 10:
-         tmp = offset | (Vdp2Regs->MPKLRA & 0xFF);
+         tmp = offset | (regs->MPKLRA & 0xFF);
          break;
       case 11:
-         tmp = offset | (Vdp2Regs->MPKLRA >> 8);
+         tmp = offset | (regs->MPKLRA >> 8);
          break;
       case 12:
-         tmp = offset | (Vdp2Regs->MPMNRA & 0xFF);
+         tmp = offset | (regs->MPMNRA & 0xFF);
          break;
       case 13:
-         tmp = offset | (Vdp2Regs->MPMNRA >> 8);
+         tmp = offset | (regs->MPMNRA >> 8);
          break;
       case 14:
-         tmp = offset | (Vdp2Regs->MPOPRA & 0xFF);
+         tmp = offset | (regs->MPOPRA & 0xFF);
          break;
       case 15:
-         tmp = offset | (Vdp2Regs->MPOPRA >> 8);
+         tmp = offset | (regs->MPOPRA >> 8);
          break;
    }
 
@@ -516,61 +516,61 @@ void FASTCALL Vdp2ParameterAPlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL Vdp2ParameterBPlaneAddr(vdp2draw_struct *info, int i)
+void FASTCALL Vdp2ParameterBPlaneAddr(vdp2draw_struct *info, int i, Vdp2* regs)
 {
-   u32 offset = (Vdp2Regs->MPOFR & 0x70) << 2;
+   u32 offset = (regs->MPOFR & 0x70) << 2;
    u32 tmp=0;
 
    // Parameter B
    switch(i)
    {
       case 0:
-         tmp = offset | (Vdp2Regs->MPABRB & 0xFF);
+         tmp = offset | (regs->MPABRB & 0xFF);
          break;
       case 1:
-         tmp = offset | (Vdp2Regs->MPABRB >> 8);
+         tmp = offset | (regs->MPABRB >> 8);
          break;
       case 2:
-         tmp = offset | (Vdp2Regs->MPCDRB & 0xFF);
+         tmp = offset | (regs->MPCDRB & 0xFF);
          break;
       case 3:
-         tmp = offset | (Vdp2Regs->MPCDRB >> 8);
+         tmp = offset | (regs->MPCDRB >> 8);
          break;
       case 4:
-         tmp = offset | (Vdp2Regs->MPEFRB & 0xFF);
+         tmp = offset | (regs->MPEFRB & 0xFF);
          break;
       case 5:
-         tmp = offset | (Vdp2Regs->MPEFRB >> 8);
+         tmp = offset | (regs->MPEFRB >> 8);
          break;
       case 6:
-         tmp = offset | (Vdp2Regs->MPGHRB & 0xFF);
+         tmp = offset | (regs->MPGHRB & 0xFF);
          break;
       case 7:
-         tmp = offset | (Vdp2Regs->MPGHRB >> 8);
+         tmp = offset | (regs->MPGHRB >> 8);
          break;
       case 8:
-         tmp = offset | (Vdp2Regs->MPIJRB & 0xFF);
+         tmp = offset | (regs->MPIJRB & 0xFF);
          break;
       case 9:
-         tmp = offset | (Vdp2Regs->MPIJRB >> 8);
+         tmp = offset | (regs->MPIJRB >> 8);
          break;
       case 10:
-         tmp = offset | (Vdp2Regs->MPKLRB & 0xFF);
+         tmp = offset | (regs->MPKLRB & 0xFF);
          break;
       case 11:
-         tmp = offset | (Vdp2Regs->MPKLRB >> 8);
+         tmp = offset | (regs->MPKLRB >> 8);
          break;
       case 12:
-         tmp = offset | (Vdp2Regs->MPMNRB & 0xFF);
+         tmp = offset | (regs->MPMNRB & 0xFF);
          break;
       case 13:
-         tmp = offset | (Vdp2Regs->MPMNRB >> 8);
+         tmp = offset | (regs->MPMNRB >> 8);
          break;
       case 14:
-         tmp = offset | (Vdp2Regs->MPOPRB & 0xFF);
+         tmp = offset | (regs->MPOPRB & 0xFF);
          break;
       case 15:
-         tmp = offset | (Vdp2Regs->MPOPRB >> 8);
+         tmp = offset | (regs->MPOPRB >> 8);
          break;
    }
 
@@ -579,21 +579,21 @@ void FASTCALL Vdp2ParameterBPlaneAddr(vdp2draw_struct *info, int i)
 
 //////////////////////////////////////////////////////////////////////////////
 
-float Vdp2ReadCoefficientMode0_2(vdp2rotationparameter_struct *parameter, u32 addr)
+float Vdp2ReadCoefficientMode0_2(vdp2rotationparameter_struct *parameter, u32 addr, u8 * ram)
 {
    s32 i;
 
    if (parameter->coefdatasize == 2)
    {
       addr &= 0x7FFFE;
-      i = T1ReadWord(Vdp2Ram, addr);
+      i = T1ReadWord(ram, addr);
       parameter->msb = (i >> 15) & 0x1;
       return (float) (signed) ((i & 0x7FFF) | (i & 0x4000 ? 0xFFFFC000 : 0x00000000)) / 1024;
    }
    else
    {
       addr &= 0x7FFFC;
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->msb = (i >> 31) & 0x1;
       return (float) (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000)) / 65536;
    }
@@ -601,21 +601,21 @@ float Vdp2ReadCoefficientMode0_2(vdp2rotationparameter_struct *parameter, u32 ad
 
 //////////////////////////////////////////////////////////////////////////////
 
-fixed32 Vdp2ReadCoefficientMode0_2FP(vdp2rotationparameterfp_struct *parameter, u32 addr)
+fixed32 Vdp2ReadCoefficientMode0_2FP(vdp2rotationparameterfp_struct *parameter, u32 addr, u8* ram)
 {
    s32 i;
 
    if (parameter->coefdatasize == 2)
    {
       addr &= 0x7FFFE;
-      i = T1ReadWord(Vdp2Ram, addr);
+      i = T1ReadWord(ram, addr);
       parameter->msb = (i >> 15) & 0x1;
       return (signed) ((i & 0x7FFF) | (i & 0x4000 ? 0xFFFFC000 : 0x00000000)) * 64;
    }
    else
    {
       addr &= 0x7FFFC;
-      i = T1ReadLong(Vdp2Ram, addr);
+      i = T1ReadLong(ram, addr);
       parameter->linescreen = (i >> 24) & 0x7F;
       parameter->msb = (i >> 31) & 0x1;
       return (signed) ((i & 0x00FFFFFF) | (i & 0x00800000 ? 0xFF800000 : 0x00000000));
