@@ -39,20 +39,26 @@
 #include "m68kcore.h"
 #include "peripheral.h"
 #include "scsp.h"
+#include "scspdsp.h"
 #include "scu.h"
 #include "sh2core.h"
 #include "smpc.h"
+#include "vidsoft.h"
 #include "vdp2.h"
 #include "yui.h"
 #include "bios.h"
 #include "movie.h"
 #include "osdcore.h"
 #ifdef HAVE_LIBSDL
- #if defined(__APPLE__) || defined(GEKKO)
-  #include <SDL/SDL.h>
+#if defined(__APPLE__) || defined(GEKKO)
+ #ifdef HAVE_LIBSDL2
+  #include <SDL2/SDL.h>
  #else
-  #include "SDL.h"
+  #include <SDL/SDL.h>
  #endif
+#else
+ #include "SDL.h"
+#endif
 #endif
 #if defined(_MSC_VER) || !defined(HAVE_SYS_TIME_H)
 #include <time.h>
@@ -84,12 +90,30 @@
     #include "gdb/stub.h"
 #endif
 
+#ifdef YAB_WANT_SSF
+#include "aosdk/ssf.h"
+#endif
+
+#include "sh7034.h"
+#include "cd_drive.h"
+#include "tsunami/yab_tsunami.h"
+#include "mpeg_card.h"
+
 //////////////////////////////////////////////////////////////////////////////
 
 yabsys_struct yabsys;
 const char *bupfilename = NULL;
 u64 tickfreq;
+//todo this ought to be in scspdsp.c
+ScspDsp scsp_dsp = { 0 };
+char ssf_track_name[256] = { 0 };
+char ssf_artist[256] = { 0 };
 
+#define SCSP_FRACTIONAL_BITS 20
+u32 saved_scsp_cycles = 0;//fixed point
+u32 saved_m68k_cycles = 0;//fixed point
+u32 saved_sh1_cycles = 0;
+u32 saved_cdd_cycles = 0;
 //////////////////////////////////////////////////////////////////////////////
 
 #ifndef NO_CLI
@@ -136,8 +160,24 @@ void YabauseChangeTiming(int freqtype) {
 
 int YabauseInit(yabauseinit_struct *init)
 {
+   tsunami_init();
+
    // Need to set this first, so init routines see it
    yabsys.UseThreads = init->usethreads;
+   yabsys.NumThreads = init->numthreads;
+   yabsys.use_cd_block_lle = init->use_cd_block_lle;
+   if (yabsys.use_cd_block_lle)
+   {
+      yabsys.use_sh2_dma_timing = 1;
+      yabsys.use_scu_dma_timing = 1;
+      yabsys.sh2_cache_enabled = 1;
+   }
+   else
+   {
+      yabsys.use_sh2_dma_timing = init->use_sh2_dma_timing;
+      yabsys.use_scu_dma_timing = init->use_scu_dma_timing;
+      yabsys.sh2_cache_enabled = init->sh2_cache_enabled;
+   }
 
    // Initialize both cpu's
    if (SH2Init(init->sh2coretype) != 0)
@@ -171,8 +211,6 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
-   MappedMemoryInit();
-
    if (VideoInit(init->vidcoretype) != 0)
    {
       YabSetError(YAB_ERR_CANNOTINIT, _("Video"));
@@ -186,7 +224,59 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
-   if (Cs2Init(init->carttype, init->cdcoretype, init->cdpath, init->mpegpath, init->netlinksetting) != 0)
+   if ((SH1Rom = T2MemoryInit(0x10000)) == NULL)
+      return -1;
+
+   if ((SH1Dram = T2MemoryInit(0x80000)) == NULL)
+      return -1;
+
+   if ((SH1MpegRom = T2MemoryInit(0x80000)) == NULL)
+      return -1;
+
+   // Initialize CD Block 
+   if (init->use_cd_block_lle)
+   {
+#if defined(SH2_DYNAREC)
+      if (init->sh1coretype == SH2CORE_DYNAREC)
+      {
+         YabSetError(YAB_ERR_CANNOTINIT, _("SH1. Dynarec core not supported for SH1 emulation."));
+         return -1;
+      }
+#endif
+
+      if (SH1Init(init->sh1coretype) != 0)
+      {
+         YabSetError(YAB_ERR_CANNOTINIT, _("SH1"));
+         return -1;
+      }
+      else
+      {
+         if (init->sh1rompath != NULL && strlen(init->sh1rompath))
+         {
+            if (LoadSH1Rom(init->sh1rompath) != 0)
+            {
+               YabSetError(YAB_ERR_FILENOTFOUND, (void *)init->sh1rompath);
+               return -2;
+            }
+         }
+         else
+         {
+            YabSetError(YAB_ERR_CANNOTINIT, _("CD Block. It needs a SH1 ROM Defined."));
+            return -1;
+         }
+
+         if (init->mpegpath != NULL && strlen(init->mpegpath))
+         {
+            if (LoadMpegRom(init->mpegpath) != 0)
+            {
+               YabSetError(YAB_ERR_FILENOTFOUND, (void *)init->mpegpath);
+               return -2;
+            }
+         }
+      }
+   }
+
+   if (Cs2Init(init->carttype, init->cdcoretype, init->cdpath, init->mpegpath, init->modemip, init->modemport) != 0)
    {
       YabSetError(YAB_ERR_CANNOTINIT, _("CS2"));
       return -1;
@@ -234,6 +324,7 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
+   MappedMemoryInit(MSH2, SSH2, SH1);
    YabauseSetVideoFormat(init->videoformattype);
    YabauseChangeTiming(CLKTYPE_26MHZ);
    yabsys.DecilineMode = 1;
@@ -259,6 +350,12 @@ int YabauseInit(yabauseinit_struct *init)
    else
       yabsys.emulatebios = 1;
 
+   if (yabsys.emulatebios && yabsys.use_cd_block_lle)
+   {
+      YabSetError(YAB_ERR_CANNOTINIT, _("CD Block. A real bios must be defined and enabled for CD Block LLE. Emulated bios not supported."));
+      return -1;
+   }
+
    yabsys.usequickload = 0;
 
    #if defined(SH2_DYNAREC)
@@ -269,6 +366,36 @@ int YabauseInit(yabauseinit_struct *init)
 
    YabauseResetNoLoad();
 
+#ifdef YAB_WANT_SSF
+
+   if (init->play_ssf && init->ssfpath != NULL && strlen(init->ssfpath))
+   {
+      if (!load_ssf((char*)init->ssfpath, init->m68kcoretype, init->sndcoretype))
+      {
+         YabSetError(YAB_ERR_FILENOTFOUND, (void *)init->ssfpath);
+
+         yabsys.playing_ssf = 0;
+
+         return -2;
+      }
+
+      yabsys.playing_ssf = 1;
+
+      get_ssf_info(1, ssf_track_name);
+      get_ssf_info(3, ssf_artist);
+
+      return 0;
+   }
+   else
+      yabsys.playing_ssf = 0;
+
+#endif
+
+   if (init->skip_load)
+   {
+	   return 0;
+   }
+
    if (yabsys.usequickload || yabsys.emulatebios)
    {
       if (YabauseQuickLoadGame() != 0)
@@ -287,11 +414,40 @@ int YabauseInit(yabauseinit_struct *init)
    GdbStubInit(MSH2, 43434);
 #endif
 
+   if (yabsys.UseThreads)
+   {
+      int num = yabsys.NumThreads < 1 ? 1 : yabsys.NumThreads;
+      VIDSoftSetVdp1ThreadEnable(num == 1 ? 0 : 1);
+      VIDSoftSetNumLayerThreads(num);
+      VIDSoftSetNumPriorityThreads(num);
+   }
+   else
+   {
+      VIDSoftSetVdp1ThreadEnable(0);
+      VIDSoftSetNumLayerThreads(0);
+      VIDSoftSetNumPriorityThreads(0);
+   }
+
+   scsp_set_use_new(init->use_new_scsp);
+
    return 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
+void YabFlushBackups(void)
+{
+   if (BupRam)
+   {
+      if (T123Save(BupRam, 0x10000, 1, bupfilename) != 0)
+         YabSetError(YAB_ERR_FILEWRITE, (void *)bupfilename);
+   }
+
+   CartFlush();
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 void YabauseDeInit(void) {
    SH2DeInit();
 
@@ -344,6 +500,8 @@ void YabauseResetNoLoad(void) {
 
    // Reset CS0 area here
    // Reset CS1 area here
+   if (yabsys.use_cd_block_lle)
+      SH2Reset(SH1);
    Cs2Reset();
    ScuReset();
    ScspReset();
@@ -352,11 +510,20 @@ void YabauseResetNoLoad(void) {
    SmpcReset();
 
    SH2PowerOn(MSH2);
+   if (yabsys.use_cd_block_lle)
+   {
+      sh1_init_func();
+      SH2PowerOn(SH1);
+   }
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 void YabauseReset(void) {
+
+   if (yabsys.playing_ssf)
+      yabsys.playing_ssf = 0;
+
    YabauseResetNoLoad();
 
    if (yabsys.usequickload || yabsys.emulatebios)
@@ -416,6 +583,11 @@ int YabauseExec(void) {
 int saved_centicycles;
 #endif
 
+static INLINE u32 get_cycles_per_line_division(u32 clock, int frames, int lines, int divisions_per_line)
+{
+   return ((u64)(clock / frames) << SCSP_FRACTIONAL_BITS) / (lines * divisions_per_line);
+}
+
 int YabauseEmulate(void) {
    int oneframeexec = 0;
 
@@ -426,21 +598,54 @@ int YabauseEmulate(void) {
 #ifndef USE_SCSP2
    unsigned int m68kcycles;       // Integral M68k cycles per call
    unsigned int m68kcenticycles;  // 1/100 M68k cycles per call
-   
+	u32 m68k_cycles_per_deciline, scsp_cycles_per_deciline, sh1_cycles_per_deciline, cdd_cycles_per_deciline;
+	int lines, frames = 0;
+
+   m68k_cycles_per_deciline = 0;
+   scsp_cycles_per_deciline = 0;
+   sh1_cycles_per_deciline = 0;
+
+   lines = 0;
+   frames = 0;
+
    if (yabsys.IsPal)
    {
-      /* 11.2896MHz / 50Hz / 313 lines / 10 calls/line = 72.20 cycles/call */
-      m68kcycles = yabsys.DecilineMode ? 72 : 722;
-      m68kcenticycles = yabsys.DecilineMode ? 20 : 0;
+      lines = 313;
+      frames = 50;
    }
    else
    {
-      /* 11.2896MHz / 60Hz / 263 lines / 10 calls/line = 71.62 cycles/call */
-      m68kcycles = yabsys.DecilineMode ? 71 : 716;
-      m68kcenticycles = yabsys.DecilineMode ? 62 : 20;
+      lines = 263;
+      frames = 60;
+   }
+
+   if (yabsys.use_cd_block_lle)
+   {
+      sh1_cycles_per_deciline = get_cycles_per_line_division(20 * 1000000, frames, lines, 10);//20mhz
+      cdd_cycles_per_deciline = get_cycles_per_line_division(1000000, frames, lines, 10);//timing is now in usec
+   }
+   if(use_new_scsp)
+   {
+      scsp_cycles_per_deciline = get_cycles_per_line_division(44100 * 512, frames, lines, 10);
+      m68k_cycles_per_deciline = get_cycles_per_line_division(44100 * 256, frames, lines, 10);
+   }
+   else
+   {
+      if (yabsys.IsPal)
+      {
+         /* 11.2896MHz / 50Hz / 313 lines / 10 calls/line = 72.20 cycles/call */
+         m68kcycles = yabsys.DecilineMode ? 72 : 722;
+         m68kcenticycles = yabsys.DecilineMode ? 20 : 0;
+      }
+      else
+      {
+         /* 11.2896MHz / 60Hz / 263 lines / 10 calls/line = 71.62 cycles/call */
+         m68kcycles = yabsys.DecilineMode ? 71 : 716;
+         m68kcenticycles = yabsys.DecilineMode ? 62 : 20;
+      }
    }
 #endif
-
+   
    DoMovie();
 
    #if defined(SH2_DYNAREC)
@@ -467,14 +672,17 @@ int YabauseEmulate(void) {
          sh2cycles = (yabsys.SH2CycleFrac >> (YABSYS_TIMING_BITS + 1)) << 1;
          yabsys.SH2CycleFrac &= ((YABSYS_TIMING_MASK << 1) | 1);
 
-         PROFILE_START("MSH2");
-         SH2Exec(MSH2, sh2cycles);
-         PROFILE_STOP("MSH2");
+         if (!yabsys.playing_ssf)
+         {
+            PROFILE_START("MSH2");
+            SH2Exec(MSH2, sh2cycles);
+            PROFILE_STOP("MSH2");
 
-         PROFILE_START("SSH2");
-         if (yabsys.IsSSH2Running)
-            SH2Exec(SSH2, sh2cycles);
-         PROFILE_STOP("SSH2");
+            PROFILE_START("SSH2");
+            if (yabsys.IsSSH2Running)
+               SH2Exec(SSH2, sh2cycles);
+            PROFILE_STOP("SSH2");
+         }
 
 #ifdef USE_SCSP2
          PROFILE_START("SCSP");
@@ -492,7 +700,7 @@ int YabauseEmulate(void) {
          }
 
          PROFILE_START("SCU");
-         ScuExec(sh2cycles / 2);
+         ScuExec(sh2cycles);
          PROFILE_STOP("SCU");
 
       } else {  // !DecilineMode
@@ -502,26 +710,31 @@ int YabauseEmulate(void) {
          yabsys.SH2CycleFrac += cyclesinc;
          sh2cycles = (yabsys.SH2CycleFrac >> (YABSYS_TIMING_BITS + 1)) << 1;
          yabsys.SH2CycleFrac &= ((YABSYS_TIMING_MASK << 1) | 1);
-
-         PROFILE_START("MSH2");
-         SH2Exec(MSH2, sh2cycles - decilinecycles);
-         PROFILE_STOP("MSH2");
-         PROFILE_START("SSH2");
-         if (yabsys.IsSSH2Running)
-            SH2Exec(SSH2, sh2cycles - decilinecycles);
-         PROFILE_STOP("SSH2");
+         if (!yabsys.playing_ssf)
+         {
+            PROFILE_START("MSH2");
+            SH2Exec(MSH2, sh2cycles - decilinecycles);
+            PROFILE_STOP("MSH2");
+            PROFILE_START("SSH2");
+            if (yabsys.IsSSH2Running)
+               SH2Exec(SSH2, sh2cycles - decilinecycles);
+            PROFILE_STOP("SSH2");
+         }
 
          PROFILE_START("hblankin");
          Vdp2HBlankIN();
          PROFILE_STOP("hblankin");
 
-         PROFILE_START("MSH2");
-         SH2Exec(MSH2, decilinecycles);
-         PROFILE_STOP("MSH2");
-         PROFILE_START("SSH2");
-         if (yabsys.IsSSH2Running)
-            SH2Exec(SSH2, decilinecycles);
-         PROFILE_STOP("SSH2");
+         if (!yabsys.playing_ssf)
+         {
+            PROFILE_START("MSH2");
+            SH2Exec(MSH2, decilinecycles);
+            PROFILE_STOP("MSH2");
+            PROFILE_START("SSH2");
+            if (yabsys.IsSSH2Running)
+               SH2Exec(SSH2, decilinecycles);
+            PROFILE_STOP("SSH2");
+         }
 
 #ifdef USE_SCSP2
          PROFILE_START("SCSP");
@@ -530,7 +743,7 @@ int YabauseEmulate(void) {
 #endif
 
          PROFILE_START("SCU");
-         ScuExec(sh2cycles / 2);
+         ScuExec(sh2cycles);
          PROFILE_STOP("SCU");
 
       }  // if (yabsys.DecilineMode)
@@ -568,6 +781,7 @@ int YabauseEmulate(void) {
             // VBlankOUT
             PROFILE_START("VDP1/VDP2");
             Vdp2VBlankOUT();
+            set_mpeg_video_irq();//guessing: set video irq once per frame
             yabsys.LineCount = 0;
             oneframeexec = 1;
             PROFILE_STOP("VDP1/VDP2");
@@ -582,8 +796,9 @@ int YabauseEmulate(void) {
       Cs2Exec(yabsys.UsecFrac >> YABSYS_TIMING_BITS);
       PROFILE_STOP("CDB");
       yabsys.UsecFrac &= YABSYS_TIMING_MASK;
-
+      
 #ifndef USE_SCSP2
+      if(!use_new_scsp)
       {
          int cycles;
 
@@ -597,7 +812,35 @@ int YabauseEmulate(void) {
          M68KExec(cycles);
          PROFILE_STOP("68K");
       }
+      else
+      {
+         u32 m68k_integer_part = 0, scsp_integer_part = 0;
+         saved_m68k_cycles += m68k_cycles_per_deciline;
+         m68k_integer_part = saved_m68k_cycles >> SCSP_FRACTIONAL_BITS;
+         M68KExec(m68k_integer_part);
+         saved_m68k_cycles -= m68k_integer_part << SCSP_FRACTIONAL_BITS;
+
+         saved_scsp_cycles += scsp_cycles_per_deciline;
+         scsp_integer_part = saved_scsp_cycles >> SCSP_FRACTIONAL_BITS;
+         new_scsp_exec(scsp_integer_part);
+         saved_scsp_cycles -= scsp_integer_part << SCSP_FRACTIONAL_BITS;
+      }
 #endif
+      if(yabsys.use_cd_block_lle)
+      {
+         u32 sh1_integer_part = 0;
+         u32 cdd_integer_part = 0;
+         saved_sh1_cycles += sh1_cycles_per_deciline;
+         sh1_integer_part = saved_sh1_cycles >> SCSP_FRACTIONAL_BITS;
+         //sh1_exec(&sh1_cxt, sh1_integer_part);
+         SH2Exec(SH1, sh1_integer_part);
+         saved_sh1_cycles -= sh1_integer_part << SCSP_FRACTIONAL_BITS;
+         
+         saved_cdd_cycles += cdd_cycles_per_deciline;
+         cdd_integer_part = saved_cdd_cycles >> SCSP_FRACTIONAL_BITS;
+         cd_drive_exec(&cdd_cxt, cdd_integer_part);
+         saved_cdd_cycles -= cdd_integer_part << SCSP_FRACTIONAL_BITS;
+      }
 
       PROFILE_STOP("Total Emulation");
    }
@@ -606,6 +849,19 @@ int YabauseEmulate(void) {
    M68KSync();
 #endif
 
+#ifdef YAB_WANT_SSF
+
+   if (yabsys.playing_ssf)
+   {
+      OSDPushMessage(OSDMSG_FPS, 1, "NAME %s", ssf_track_name);
+      OSDPushMessage(OSDMSG_STATUS, 1, "ARTIST %s", ssf_artist);
+   }
+
+#endif
+
+   //flush tsunami output once per frame
+   tsunami_flush();
+
    return 0;
 }
 
@@ -614,32 +870,30 @@ int YabauseEmulate(void) {
 void YabauseStartSlave(void) {
    if (yabsys.emulatebios)
    {
-      CurrentSH2 = SSH2;
-      MappedMemoryWriteLong(0xFFFFFFE0, 0xA55A03F1); // BCR1
-      MappedMemoryWriteLong(0xFFFFFFE4, 0xA55A00FC); // BCR2
-      MappedMemoryWriteLong(0xFFFFFFE8, 0xA55A5555); // WCR
-      MappedMemoryWriteLong(0xFFFFFFEC, 0xA55A0070); // MCR
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFE0, 0xA55A03F1); // BCR1
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFE4, 0xA55A00FC); // BCR2
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFE8, 0xA55A5555); // WCR
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFEC, 0xA55A0070); // MCR
 
-      MappedMemoryWriteWord(0xFFFFFEE0, 0x0000); // ICR
-      MappedMemoryWriteWord(0xFFFFFEE2, 0x0000); // IPRA
-      MappedMemoryWriteWord(0xFFFFFE60, 0x0F00); // VCRWDT
-      MappedMemoryWriteWord(0xFFFFFE62, 0x6061); // VCRA
-      MappedMemoryWriteWord(0xFFFFFE64, 0x6263); // VCRB
-      MappedMemoryWriteWord(0xFFFFFE66, 0x6465); // VCRC
-      MappedMemoryWriteWord(0xFFFFFE68, 0x6600); // VCRD
-      MappedMemoryWriteWord(0xFFFFFEE4, 0x6869); // VCRWDT
-      MappedMemoryWriteLong(0xFFFFFFA8, 0x0000006C); // VCRDMA1
-      MappedMemoryWriteLong(0xFFFFFFA0, 0x0000006D); // VCRDMA0
-      MappedMemoryWriteLong(0xFFFFFF0C, 0x0000006E); // VCRDIV
-      MappedMemoryWriteLong(0xFFFFFE10, 0x00000081); // TIER
-      CurrentSH2 = MSH2;
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFEE0, 0x0000); // ICR
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFEE2, 0x0000); // IPRA
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFE60, 0x0F00); // VCRWDT
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFE62, 0x6061); // VCRA
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFE64, 0x6263); // VCRB
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFE66, 0x6465); // VCRC
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFE68, 0x6600); // VCRD
+      SSH2->MappedMemoryWriteWord(SSH2, 0xFFFFFEE4, 0x6869); // VCRWDT
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFA8, 0x0000006C); // VCRDMA1
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFFA0, 0x0000006D); // VCRDMA0
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFF0C, 0x0000006E); // VCRDIV
+      SSH2->MappedMemoryWriteLong(SSH2, 0xFFFFFE10, 0x00000081); // TIER
 
       SH2GetRegisters(SSH2, &SSH2->regs);
-      SSH2->regs.R[15] = 0x06001000;
+      SSH2->regs.R[15] = Cs2GetSlaveStackAdress();
       SSH2->regs.VBR = 0x06000400;
-      SSH2->regs.PC = MappedMemoryReadLong(0x06000250);
-      if (MappedMemoryReadLong(0x060002AC) != 0)
-         SSH2->regs.R[15] = MappedMemoryReadLong(0x060002AC);
+      SSH2->regs.PC = MSH2->MappedMemoryReadLong(MSH2, 0x06000250);
+      if (MSH2->MappedMemoryReadLong(MSH2, 0x060002AC) != 0)
+         SSH2->regs.R[15] = MSH2->MappedMemoryReadLong(MSH2, 0x060002AC);
       SH2SetRegisters(SSH2, &SSH2->regs);
    }
    else
@@ -717,46 +971,46 @@ void YabauseSpeedySetup(void)
       // Setup the vector table area, etc.(all bioses have it at 0x00000600-0x00000810)
       for (i = 0; i < 0x210; i+=4)
       {
-         data = MappedMemoryReadLong(0x00000600+i);
-         MappedMemoryWriteLong(0x06000000+i, data);
+         data = MappedMemoryReadLongNocache(MSH2, 0x00000600+i);
+         MappedMemoryWriteLongNocache(MSH2, 0x06000000+i, data);
       }
 
       // Setup the bios function pointers, etc.(all bioses have it at 0x00000820-0x00001100)
       for (i = 0; i < 0x8E0; i+=4)
       {
-         data = MappedMemoryReadLong(0x00000820+i);
-         MappedMemoryWriteLong(0x06000220+i, data);
+         data = MappedMemoryReadLongNocache(MSH2, 0x00000820+i);
+         MappedMemoryWriteLongNocache(MSH2, 0x06000220+i, data);
       }
 
       // I'm not sure this is really needed
       for (i = 0; i < 0x700; i+=4)
       {
-         data = MappedMemoryReadLong(0x00001100+i);
-         MappedMemoryWriteLong(0x06001100+i, data);
+         data = MappedMemoryReadLongNocache(MSH2, 0x00001100+i);
+         MappedMemoryWriteLongNocache(MSH2, 0x06001100+i, data);
       }
 
       // Fix some spots in 0x06000210-0x0600032C area
-      MappedMemoryWriteLong(0x06000234, 0x000002AC);
-      MappedMemoryWriteLong(0x06000238, 0x000002BC);
-      MappedMemoryWriteLong(0x0600023C, 0x00000350);
-      MappedMemoryWriteLong(0x06000240, 0x32524459);
-      MappedMemoryWriteLong(0x0600024C, 0x00000000);
-      MappedMemoryWriteLong(0x06000268, MappedMemoryReadLong(0x00001344));
-      MappedMemoryWriteLong(0x0600026C, MappedMemoryReadLong(0x00001348));
-      MappedMemoryWriteLong(0x0600029C, MappedMemoryReadLong(0x00001354));
-      MappedMemoryWriteLong(0x060002C4, MappedMemoryReadLong(0x00001104));
-      MappedMemoryWriteLong(0x060002C8, MappedMemoryReadLong(0x00001108));
-      MappedMemoryWriteLong(0x060002CC, MappedMemoryReadLong(0x0000110C));
-      MappedMemoryWriteLong(0x060002D0, MappedMemoryReadLong(0x00001110));
-      MappedMemoryWriteLong(0x060002D4, MappedMemoryReadLong(0x00001114));
-      MappedMemoryWriteLong(0x060002D8, MappedMemoryReadLong(0x00001118));
-      MappedMemoryWriteLong(0x060002DC, MappedMemoryReadLong(0x0000111C));
-      MappedMemoryWriteLong(0x06000328, 0x000004C8);
-      MappedMemoryWriteLong(0x0600032C, 0x00001800);
+      MappedMemoryWriteLongNocache(MSH2, 0x06000234, 0x000002AC);
+      MappedMemoryWriteLongNocache(MSH2, 0x06000238, 0x000002BC);
+      MappedMemoryWriteLongNocache(MSH2, 0x0600023C, 0x00000350);
+      MappedMemoryWriteLongNocache(MSH2, 0x06000240, 0x32524459);
+      MappedMemoryWriteLongNocache(MSH2, 0x0600024C, 0x00000000);
+      MappedMemoryWriteLongNocache(MSH2, 0x06000268, MappedMemoryReadLongNocache(MSH2, 0x00001344));
+      MappedMemoryWriteLongNocache(MSH2, 0x0600026C, MappedMemoryReadLongNocache(MSH2, 0x00001348));
+      MappedMemoryWriteLongNocache(MSH2, 0x0600029C, MappedMemoryReadLongNocache(MSH2, 0x00001354));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002C4, MappedMemoryReadLongNocache(MSH2, 0x00001104));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002C8, MappedMemoryReadLongNocache(MSH2, 0x00001108));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002CC, MappedMemoryReadLongNocache(MSH2, 0x0000110C));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002D0, MappedMemoryReadLongNocache(MSH2, 0x00001110));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002D4, MappedMemoryReadLongNocache(MSH2, 0x00001114));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002D8, MappedMemoryReadLongNocache(MSH2, 0x00001118));
+      MappedMemoryWriteLongNocache(MSH2, 0x060002DC, MappedMemoryReadLongNocache(MSH2, 0x0000111C));
+      MappedMemoryWriteLongNocache(MSH2, 0x06000328, 0x000004C8);
+      MappedMemoryWriteLongNocache(MSH2, 0x0600032C, 0x00001800);
 
       // Fix SCU interrupts
       for (i = 0; i < 0x80; i+=4)
-         MappedMemoryWriteLong(0x06000A00+i, 0x0600083C);
+         MappedMemoryWriteLongNocache(MSH2, 0x06000A00+i, 0x0600083C);
    }
 
    // Set the cpu's, etc. to sane states
@@ -891,12 +1145,12 @@ int YabauseQuickLoadGame(void)
          if (size >= 2048)
          {
             for (i2 = 0; i2 < 2048; i2++)
-               MappedMemoryWriteByte(0x06002000 + (i * 0x800) + i2, buffer[i2]);
+               MappedMemoryWriteByteNocache(MSH2, 0x06002000 + (i * 0x800) + i2, buffer[i2]);
          }
          else
          {
             for (i2 = 0; i2 < size; i2++)
-               MappedMemoryWriteByte(0x06002000 + (i * 0x800) + i2, buffer[i2]);
+               MappedMemoryWriteByteNocache(MSH2, 0x06002000 + (i * 0x800) + i2, buffer[i2]);
          }
 
          size -= 2048;
@@ -963,12 +1217,12 @@ int YabauseQuickLoadGame(void)
          if (size >= 2048)
          {
             for (i2 = 0; i2 < 2048; i2++)
-               MappedMemoryWriteByte(addr + (i * 0x800) + i2, buffer[i2]);
+               MappedMemoryWriteByteNocache(MSH2, addr + (i * 0x800) + i2, buffer[i2]);
          }
          else
          {
             for (i2 = 0; i2 < size; i2++)
-               MappedMemoryWriteByte(addr + (i * 0x800) + i2, buffer[i2]);
+               MappedMemoryWriteByteNocache(MSH2, addr + (i * 0x800) + i2, buffer[i2]);
          }
 
          size -= 2048;
@@ -985,6 +1239,7 @@ int YabauseQuickLoadGame(void)
       // Now setup SH2 registers to start executing at ip code
       SH2GetRegisters(MSH2, &MSH2->regs);
       MSH2->regs.PC = 0x06002E00;
+      MSH2->regs.R[15] = Cs2GetMasterStackAdress();
       SH2SetRegisters(MSH2, &MSH2->regs);
    }
    else
