@@ -23,10 +23,12 @@
 #define YABAUSE_H
 
 #include "core.h"
+#include "sh2core.h"
 
 typedef struct
 {
    int percoretype;
+   int sh1coretype;
    int sh2coretype;
    int vidcoretype;
    int sndcoretype;
@@ -36,16 +38,27 @@ typedef struct
    u8 regionid;
    const char *biospath;
    const char *cdpath;
+   const char *ssfpath;
    const char *buppath;
    const char *mpegpath;
    const char *cartpath;
-   const char *netlinksetting;
+   const char *modemip;
+   const char *modemport;
+   const char *sh1rompath;
    int videoformattype;
    int frameskip;
    int clocksync;  // 1 = sync internal clock to emulation, 0 = realtime clock
    u32 basetime;   // Initial time in clocksync mode (0 = start w/ system time)
    int usethreads;
+   int numthreads;
    int osdcoretype;
+   int skip_load;//skip loading in YabauseInit so tests can be run without a bios
+   int play_ssf;
+   int use_new_scsp;
+   int use_cd_block_lle;
+   int use_sh2_dma_timing;
+   int use_scu_dma_timing;
+   int sh2_cache_enabled;
 } yabauseinit_struct;
 
 #define CLKTYPE_26MHZ           0
@@ -60,6 +73,7 @@ void print_usage(const char *program_name);
 
 void YabauseChangeTiming(int freqtype);
 int YabauseInit(yabauseinit_struct *init);
+void YabFlushBackups(void);
 void YabauseDeInit(void);
 void YabauseSetDecilineMode(int on);
 void YabauseResetNoLoad(void);
@@ -90,11 +104,17 @@ typedef struct
    int CurSH2FreqType;
    int IsPal;
    u8 UseThreads;
+   int NumThreads;
    u8 IsSSH2Running;
    u64 OneFrameTime;
    u64 tickfreq;
    int emulatebios;
    int usequickload;
+   int playing_ssf;
+   int use_cd_block_lle;
+   int use_sh2_dma_timing;
+   int use_scu_dma_timing;
+   int sh2_cache_enabled;
 } yabsys_struct;
 
 extern yabsys_struct yabsys;
