@@ -18,44 +18,65 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#ifdef HAVE_LIBGL
-#ifdef _WIN32
-#include <windows.h>
-#include <GL/gl.h>
-#include "glext.h"
-#endif
+#if defined(HAVE_LIBGL) || defined(__ANDROID__)
 
-#ifdef HAVE_LIBSDL
- #ifdef __APPLE__
-  #include <SDL/SDL.h>
- #else
-  #include "SDL.h"
- #endif
-#endif
-#ifndef _arch_dreamcast
-    #ifdef __APPLE__
-        #include <OpenGL/gl.h>
+#if defined(__ANDROID__)
+    #include <GLES3/gl3.h>
+    #include <GLES3/gl3ext.h>
+    #include <EGL/egl.h>
+
+#elif defined(_WIN32)
+
+#include <windows.h>
+  #if defined(_USEGLEW_)
+    #include <GL/glew.h>
+    #include <GL/gl.h>
+    #include "glext.h"
+#else
+    #include <GL/gl.h>
+    #include "glext.h"
+    extern PFNGLACTIVETEXTUREPROC glActiveTexture;
+  #endif
+
+#elif  defined(__APPLE__)
+    #include <OpenGL/gl.h>
+    #include <OpenGL/gl3.h>
+
+#else // Linux?
+    #if defined(_OGLES3_)||defined(_OGL3_)
+        #define GL_GLEXT_PROTOTYPES 1
+        #define GLX_GLXEXT_PROTOTYPES 1
+        #include <GL/glew.h>
+        #include <GL/gl.h>
     #else
         #include <GL/gl.h>
     #endif
 #endif
+
+#include <stdarg.h>
 #include <string.h>
 
 #ifndef YGL_H
 #define YGL_H
 
 #include "core.h"
+#include "threads.h"
 #include "vidshared.h"
 
 typedef struct {
-	int vertices[8];
-	unsigned int w;
-	unsigned int h;
+	float vertices[8];
+	int w;
+	int h;
 	int flip;
 	int priority;
 	int dst;
     int uclipmode;
     int blendmode;
+    s32 cor;
+    s32 cog;
+    s32 cob;
+    int linescreen;
+
 } YglSprite;
 
 typedef struct {
@@ -87,7 +108,8 @@ void YglTMAllocate(YglTexture *, unsigned int, unsigned int, unsigned int *, uns
 
 enum
 {
-   PG_NORMAL=0,
+   PG_NORMAL=1,
+   PG_VDP1_NORMAL,
    PG_VFP1_GOURAUDSAHDING,
    PG_VFP1_STARTUSERCLIP,
    PG_VFP1_ENDUSERCLIP,
@@ -96,14 +118,19 @@ enum
    PG_VDP2_ADDBLEND,
    PG_VDP2_DRAWFRAMEBUFF,    
    PG_VDP2_STARTWINDOW,
-   PG_VDP2_ENDWINDOW,    
+   PG_VDP2_ENDWINDOW,
+   PG_WINDOW,
+   PG_LINECOLOR_INSERT,
+   PG_VDP2_DRAWFRAMEBUFF_LINECOLOR,
+   PG_VDP2_DRAWFRAMEBUFF_ADDCOLOR,
    PG_MAX,
 };
 
 typedef struct {
    int prgid;
    GLuint prg;
-   int * quads;
+   GLuint vertexBuffer;
+   float * quads;
    float * textcoords;
    float * vertexAttribute;
    int currentQuad;
@@ -113,6 +140,14 @@ typedef struct {
    short ux1,uy1,ux2,uy2;
    int blendmode;
    int bwin0,logwin0,bwin1,logwin1,winmode;
+   GLuint vertexp;
+   GLuint texcoordp;
+   GLuint mtxModelView;
+   GLuint mtxTexture;
+   GLuint color_offset;
+   GLuint tex0;
+   GLuint tex1;
+   float color_offset_val[4];
    int (*setupUniform)(void *);
    int (*cleanupUniform)(void *);
 } YglProgram;
@@ -126,12 +161,24 @@ typedef struct {
    YglProgram * prg;
 } YglLevel;
 
+typedef struct
+{
+    GLfloat   m[4][4];
+} YglMatrix;
+
 typedef struct {
    GLuint texture;
+   GLuint pixelBufferID;
    int st;
+   char message[512];
+   int msglength;
    unsigned int width;
    unsigned int height;
    unsigned int depth;
+
+   float clear_r;
+   float clear_g;
+   float clear_b;
    
    // VDP1 Info
    int vdp1_maxpri;
@@ -141,9 +188,15 @@ typedef struct {
    int rwidth;
    int rheight;
    int drawframe;
-   GLuint rboid;
+   int readframe;
+   GLuint rboid_depth;
+   GLuint rboid_stencil;
    GLuint vdp1fbo;
    GLuint vdp1FrameBuff[2];
+   GLuint smallfbo;
+   GLuint smallfbotex;
+   GLuint vdp1pixelBufferID;
+   void * pFrameBuffer;
 
    // Message Layer
    int msgwidth;
@@ -157,7 +210,18 @@ typedef struct {
    int win1v[512*4];
    int win1_vertexcnt;
 
+   YglMatrix mtxModelView;
+   YglMatrix mtxTexture;
+
+   YglProgram windowpg;
+   YglProgram renderfb;
+
    YglLevel * levels;
+   
+   u32 lincolor_tex;
+   u32 linecolor_pbo;
+   u32 * lincolor_buf;
+
 }  Ygl;
 
 extern Ygl * _Ygl;
@@ -167,6 +231,8 @@ int YglGLInit(int, int);
 int YglInit(int, int, unsigned int);
 void YglDeInit(void);
 float * YglQuad(YglSprite *, YglTexture *,YglCache * c);
+void YglQuadOffset(YglSprite * input, YglTexture * output, YglCache * c, int cx, int cy, float sx, float sy);
+void YglCachedQuadOffset(YglSprite * input, YglCache * cache, int cx, int cy, float sx, float sy);
 void YglCachedQuad(YglSprite *, YglCache *);
 void YglRender(void);
 void YglReset(void);
@@ -174,10 +240,12 @@ void YglShowTexture(void);
 void YglChangeResolution(int, int);
 void YglCacheQuadGrowShading(YglSprite * input, float * colors, YglCache * cache);
 int YglQuadGrowShading(YglSprite * input, YglTexture * output, float * colors,YglCache * c);
-
+void YglSetClearColor(float r, float g, float b);
 void YglStartWindow( vdp2draw_struct * info, int win0, int logwin0, int win1, int logwin1, int mode );
 void YglEndWindow( vdp2draw_struct * info );
 
+void YglCacheInit(void);
+void YglCacheDeInit(void);
 int YglIsCached(u32,YglCache *);
 void YglCacheAdd(u32,YglCache *);
 void YglCacheReset(void);
@@ -185,23 +253,41 @@ void YglCacheReset(void);
 // 0.. no belnd, 1.. Alpha, 2.. Add 
 int YglSetLevelBlendmode( int pri, int mode );
 
-int Ygl_uniformVDP2DrawFramebuffer( float from, float to , float * offsetcol );
+void Ygl_uniformVDP2DrawFramebuffer_linecolor(void * p, float from, float to, float * offsetcol);
+int Ygl_uniformVDP2DrawFramebuffer_addcolor(void * p, float from, float to, float * offsetcol);
+void Ygl_uniformVDP2DrawFramebuffer( void * p,float from, float to , float * offsetcol );
 
 void YglNeedToUpdateWindow();
 
+void YglScalef(YglMatrix *result, GLfloat sx, GLfloat sy, GLfloat sz);
+void YglTranslatef(YglMatrix *result, GLfloat tx, GLfloat ty, GLfloat tz);
+void YglRotatef(YglMatrix *result, GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
+void YglFrustum(YglMatrix *result, float left, float right, float bottom, float top, float nearZ, float farZ);
+void YglPerspective(YglMatrix *result, float fovy, float aspect, float nearZ, float farZ);
+void YglOrtho(YglMatrix *result, float left, float right, float bottom, float top, float nearZ, float farZ);
+void YglLoadIdentity(YglMatrix *result);
+void YglMatrixMultiply(YglMatrix *result, YglMatrix *srcA, YglMatrix *srcB);
+
+int YglInitVertexBuffer( int initsize );
+void YglDeleteVertexBuffer();
+int YglUnMapVertexBuffer();
+int YglMapVertexBuffer();
+int YglUserDirectVertexBuffer();
+int YglUserVertexBuffer();
+int YglGetVertexBuffer( int size, void ** vpos, void **tcpos, void **vapos );
+int YglExpandVertexBuffer( int addsize, void ** vpos, void **tcpos, void **vapos );
+intptr_t YglGetOffset( void* address );
+int YglBlitFramebuffer(u32 srcTexture, u32 targetFbo, float w, float h);
+
+void YglRenderVDP1(void);
+u32 * YglGetLineColorPointer();
+void YglSetLineColor(u32 * pbuf, int size);
+
+int Ygl_uniformWindow(void * p );
 int YglProgramInit();
 int YglProgramChange( YglLevel * level, int prgid );
 
-#if 1  // Does anything need this?  It breaks a bunch of prototypes if
-       // GLchar is typedef'd instead of #define'd  --AC
-#ifndef GLchar
-#define GLchar GLbyte
-#endif
-#endif  // 0
-
-#ifdef __APPLE__
-
-#else
+#if !defined(__APPLE__) && !defined(__ANDROID__) && !defined(_USEGLEW_) && !defined(_OGLES3_)
 
 extern GLuint (STDCALL *glCreateProgram)(void);
 extern GLuint (STDCALL *glCreateShader)(GLenum);
@@ -249,11 +335,8 @@ extern PFNGLUNIFORM4FPROC glUniform4f;
 extern PFNGLUNIFORM1FPROC glUniform1f;
 extern PFNGLUNIFORMMATRIX4FVPROC glUniformMatrix4fv;
 
-#ifdef WIN32
-extern PFNGLACTIVETEXTUREPROC glActiveTexture;
-#endif
+#endif // !defined(__APPLE__) && !defined(__ANDROID__) && !defined(_USEGLEW_)
 
-#endif
+#endif // YGL_H
 
-#endif
-#endif
+#endif // defined(HAVE_LIBGL) || defined(__ANDROID__)
